@@ -2,7 +2,7 @@
  * libmpeg2.c: mpeg2 video decoder module making use of libmpeg2.
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: libmpeg2.c,v 1.8 2003/04/07 17:35:01 gbazin Exp $
+ * $Id: libmpeg2.c,v 1.22 2003/06/10 23:01:40 massiot Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -34,6 +34,8 @@
 
 #include <mpeg2dec/mpeg2.h>
 
+#include "vout_synchro.h"
+
 /* Aspect ratio (ISO/IEC 13818-2 section 6.3.3, table 6-3) */
 #define AR_SQUARE_PICTURE       1                           /* square pixels */
 #define AR_3_4_PICTURE          2                        /* 3:4 picture (TV) */
@@ -50,6 +52,7 @@ typedef struct dec_thread_t
      */
     mpeg2dec_t          *p_mpeg2dec;
     const mpeg2_info_t  *p_info;
+    vlc_bool_t          b_skip;
 
     /*
      * Input properties
@@ -60,11 +63,18 @@ typedef struct dec_thread_t
     mtime_t          i_previous_pts;
     mtime_t          i_current_pts;
     mtime_t          i_period_remainder;
+    int              i_current_rate;
+    picture_t *      p_picture_to_destroy;
+    vlc_bool_t       b_garbage_pic;
+    vlc_bool_t       b_after_sequence_header; /* is it the next frame after
+                                               * the sequence header ?    */
+    vlc_bool_t       b_slice_i;             /* intra-slice refresh stream */
 
     /*
      * Output properties
      */
     vout_thread_t *p_vout;
+    vout_synchro_t *p_synchro;
 
 } dec_thread_t;
 
@@ -102,6 +112,7 @@ static int OpenDecoder( vlc_object_t *p_this )
     p_fifo->pf_run = RunDecoder;
     return VLC_SUCCESS;
 }
+
 /*****************************************************************************
  * RunDecoder: the libmpeg2 decoder
  *****************************************************************************/
@@ -127,11 +138,16 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
     p_dec->p_pes      = NULL;
     p_dec->p_vout     = NULL;
     p_dec->p_mpeg2dec = NULL;
+    p_dec->p_synchro  = NULL;
     p_dec->p_info     = NULL;
     p_dec->i_pts      = mdate() + DEFAULT_PTS_DELAY;
     p_dec->i_current_pts  = 0;
     p_dec->i_previous_pts = 0;
     p_dec->i_period_remainder = 0;
+    p_dec->p_picture_to_destroy = NULL;
+    p_dec->b_garbage_pic = 0;
+    p_dec->b_slice_i  = 0;
+    p_dec->b_skip     = 0;
 
     /* Initialize decoder */
     p_dec->p_mpeg2dec = mpeg2_init();
@@ -163,6 +179,45 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
                     break;
                 }
 
+                if( p_dec->p_pes->b_discontinuity && p_dec->p_synchro 
+                     && p_dec->p_info->sequence->width != (unsigned)-1 )
+                {
+                    vout_SynchroReset( p_dec->p_synchro );
+                    if ( p_dec->p_info->current_fbuf != NULL
+                          && p_dec->p_info->current_fbuf->id != NULL )
+                    {
+                        p_dec->b_garbage_pic = 1;
+                        p_pic = p_dec->p_info->current_fbuf->id;
+                    }
+                    else
+                    {
+                        uint8_t *buf[3];
+                        buf[0] = buf[1] = buf[2] = NULL;
+                        if( (p_pic = GetNewPicture( p_dec, buf )) == NULL )
+                            break;
+                        mpeg2_set_buf( p_dec->p_mpeg2dec, buf, p_pic );
+                    }
+                    p_dec->p_picture_to_destroy = p_pic;
+
+                    memset( p_pic->p[0].p_pixels, 0,
+                            p_dec->p_info->sequence->width
+                             * p_dec->p_info->sequence->height );
+                    memset( p_pic->p[1].p_pixels, 0x80,
+                            p_dec->p_info->sequence->width
+                             * p_dec->p_info->sequence->height / 4 );
+                    memset( p_pic->p[2].p_pixels, 0x80,
+                            p_dec->p_info->sequence->width
+                             * p_dec->p_info->sequence->height / 4 );
+
+                    if ( p_dec->b_slice_i )
+                    {
+                        vout_SynchroNewPicture( p_dec->p_synchro,
+                            I_CODING_TYPE, 2, 0, 0, p_dec->i_current_rate );
+                        vout_SynchroDecode( p_dec->p_synchro );
+                        vout_SynchroEnd( p_dec->p_synchro, I_CODING_TYPE, 0 );
+                    }
+                }
+
                 if( p_dec->p_pes->i_pts )
                 {
                     mpeg2_pts( p_dec->p_mpeg2dec,
@@ -170,6 +225,8 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
                     p_dec->i_previous_pts = p_dec->i_current_pts;
                     p_dec->i_current_pts = p_dec->p_pes->i_pts;
                 }
+
+                p_dec->i_current_rate = p_dec->p_pes->i_rate;
                 p_data = p_dec->p_pes->p_first;
             }
 
@@ -187,6 +244,7 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
         {
             /* Initialize video output */
             uint8_t *buf[3];
+            buf[0] = buf[1] = buf[2] = NULL;
 
             /* Check whether the input gives a particular aspect ratio */
             if( p_dec->p_fifo->p_demux_data
@@ -215,9 +273,9 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
             else
             {
                 /* Use the value provided in the MPEG sequence header */
-                i_aspect = ((uint64_t)p_dec->p_info->sequence->width) *
+                i_aspect = ((uint64_t)p_dec->p_info->sequence->display_width) *
                     p_dec->p_info->sequence->pixel_width * VOUT_ASPECT_FACTOR /
-                    p_dec->p_info->sequence->height /
+                    p_dec->p_info->sequence->display_height /
                     p_dec->p_info->sequence->pixel_height;
             }
 
@@ -228,29 +286,111 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
                                           p_dec->p_info->sequence->height,
                                           i_chroma, i_aspect );
 
+            msg_Dbg( p_dec->p_fifo, "%dx%d, aspect %d, %u.%03u fps",
+                     p_dec->p_info->sequence->width,
+                     p_dec->p_info->sequence->height, i_aspect,
+                     (u32)((u64)1001000000 * 27 / p_dec->p_info->sequence->frame_period / 1001),
+                     (u32)((u64)1001000000 * 27 / p_dec->p_info->sequence->frame_period % 1001) );
+
             mpeg2_custom_fbuf( p_dec->p_mpeg2dec, 1 );
 
             /* Set the first 2 reference frames */
+            mpeg2_set_buf( p_dec->p_mpeg2dec, buf, NULL );
+
             if( (p_pic = GetNewPicture( p_dec, buf )) == NULL ) break;
+            memset( p_pic->p[0].p_pixels, 0,
+                    p_dec->p_info->sequence->width
+                     * p_dec->p_info->sequence->height );
+            memset( p_pic->p[1].p_pixels, 0x80,
+                    p_dec->p_info->sequence->width
+                     * p_dec->p_info->sequence->height / 4 );
+            memset( p_pic->p[2].p_pixels, 0x80,
+                    p_dec->p_info->sequence->width
+                     * p_dec->p_info->sequence->height / 4 );
             mpeg2_set_buf( p_dec->p_mpeg2dec, buf, p_pic );
-            if( (p_pic = GetNewPicture( p_dec, buf )) == NULL ) break;
-            mpeg2_set_buf( p_dec->p_mpeg2dec, buf, p_pic );
+            /* This picture will never go through display_picture. */
+            vout_DatePicture( p_dec->p_vout, p_pic, 0 );
+            vout_DisplayPicture( p_dec->p_vout, p_pic );
+            /* For some reason, libmpeg2 will put this pic twice in
+             * discard_picture. This can be considered a bug in libmpeg2. */
+            vout_LinkPicture( p_dec->p_vout, p_pic );
+
+            if ( p_dec->p_synchro )
+            {
+                vout_SynchroRelease( p_dec->p_synchro );
+            }
+            p_dec->p_synchro = vout_SynchroInit( p_dec->p_fifo, p_dec->p_vout,
+                (u32)((u64)1001000000 * 27 / p_dec->p_info->sequence->frame_period) );
+            p_dec->b_after_sequence_header = 1;
         }
         break;
 
+        case STATE_PICTURE_2ND:
+            vout_SynchroNewPicture( p_dec->p_synchro,
+                p_dec->p_info->current_picture->flags & PIC_MASK_CODING_TYPE,
+                p_dec->p_info->current_picture->nb_fields,
+                0, 0,
+                p_dec->i_current_rate );
+
+            if ( p_dec->b_skip )
+            {
+                vout_SynchroTrash( p_dec->p_synchro );
+            }
+            else
+            {
+                vout_SynchroDecode( p_dec->p_synchro );
+            }
+            break;
+
         case STATE_PICTURE:
         {
             uint8_t *buf[3];
+            buf[0] = buf[1] = buf[2] = NULL;
 
-            if( (p_pic = GetNewPicture( p_dec, buf )) == NULL ) break;
-            mpeg2_set_buf( p_dec->p_mpeg2dec, buf, p_pic );
-
-            /* Store the date for the picture */
-            if( p_dec->p_info->current_picture->flags & PIC_FLAG_PTS )
+            if ( p_dec->b_after_sequence_header
+                  && ((p_dec->p_info->current_picture->flags
+                        & PIC_MASK_CODING_TYPE)
+                       == PIC_FLAG_CODING_TYPE_P) )
             {
-                p_pic->date = ( p_dec->p_info->current_picture->pts ==
-                                (uint32_t)p_dec->i_current_pts ) ?
-                              p_dec->i_current_pts : p_dec->i_previous_pts;
+                /* Intra-slice refresh. Simulate a blank I picture. */
+                msg_Dbg( p_dec->p_fifo, "intra-slice refresh stream" );
+                vout_SynchroNewPicture( p_dec->p_synchro,
+                    I_CODING_TYPE, 2, 0, 0, p_dec->i_current_rate );
+                vout_SynchroDecode( p_dec->p_synchro );
+                vout_SynchroEnd( p_dec->p_synchro, I_CODING_TYPE, 0 );
+                p_dec->b_slice_i = 1;
+            }
+            p_dec->b_after_sequence_header = 0;
+
+            vout_SynchroNewPicture( p_dec->p_synchro,
+                p_dec->p_info->current_picture->flags & PIC_MASK_CODING_TYPE,
+                p_dec->p_info->current_picture->nb_fields,
+                (p_dec->p_info->current_picture->flags & PIC_FLAG_PTS) ?
+                    ( (p_dec->p_info->current_picture->pts ==
+                                (uint32_t)p_dec->i_current_pts) ?
+                              p_dec->i_current_pts : p_dec->i_previous_pts ) : 0,
+                0,
+                p_dec->i_current_rate );
+
+            if ( !(p_dec->b_slice_i
+                   && ((p_dec->p_info->current_picture->flags
+                         & PIC_MASK_CODING_TYPE) == P_CODING_TYPE))
+                   && !vout_SynchroChoose( p_dec->p_synchro,
+                              p_dec->p_info->current_picture->flags
+                                & PIC_MASK_CODING_TYPE ) )
+            {
+                mpeg2_skip( p_dec->p_mpeg2dec, 1 );
+                p_dec->b_skip = 1;
+                vout_SynchroTrash( p_dec->p_synchro );
+                mpeg2_set_buf( p_dec->p_mpeg2dec, buf, NULL );
+            }
+            else
+            {
+                mpeg2_skip( p_dec->p_mpeg2dec, 0 );
+                p_dec->b_skip = 0;
+                vout_SynchroDecode( p_dec->p_synchro );
+                if( (p_pic = GetNewPicture( p_dec, buf )) == NULL ) break;
+                mpeg2_set_buf( p_dec->p_mpeg2dec, buf, p_pic );
             }
         }
         break;
@@ -262,45 +402,82 @@ static int RunDecoder( decoder_fifo_t *p_fifo )
             {
                 p_pic = (picture_t *)p_dec->p_info->display_fbuf->id;
 
-                /* Date the new picture */
-                if( p_dec->p_info->display_picture->flags & PIC_FLAG_PTS )
+                vout_SynchroEnd( p_dec->p_synchro,
+                            p_dec->p_info->display_picture->flags
+                             & PIC_MASK_CODING_TYPE,
+                            p_dec->b_garbage_pic );
+                p_dec->b_garbage_pic = 0;
+                vout_DisplayPicture( p_dec->p_vout, p_pic );
+
+                if ( p_dec->p_picture_to_destroy != p_pic )
                 {
-                    p_dec->i_pts = p_pic->date;
-                    p_dec->i_period_remainder = 0;
+                    vout_DatePicture( p_dec->p_vout, p_pic,
+                        vout_SynchroDate( p_dec->p_synchro ) );
                 }
                 else
                 {
-                    p_dec->i_pts += ( (p_dec->p_info->sequence->frame_period +
-                                       p_dec->i_period_remainder) / 27 );
-                    p_dec->i_period_remainder =
-                        p_dec->p_info->sequence->frame_period +
-                        p_dec->i_period_remainder -
-                        ( p_dec->p_info->sequence->frame_period +
-                          p_dec->i_period_remainder ) / 27 * 27;
-                }
-                vout_DatePicture( p_dec->p_vout, p_pic, p_dec->i_pts );
-
-                vout_DisplayPicture( p_dec->p_vout, p_pic );
-
-                /* Handle pulldown by adding some delay to the pts of the next
-                 * picture. */
-                if( p_dec->p_info->display_picture->nb_fields > 2 )
-                {
-                    int i_repeat_fields =
-                        p_dec->p_info->display_picture->nb_fields - 2;
-
-                    p_dec->i_pts += ( (p_dec->p_info->sequence->frame_period +
-                                       p_dec->i_period_remainder)
-                                      / 27 / 2 * i_repeat_fields );
-                    p_dec->i_period_remainder =
-                        p_dec->p_info->sequence->frame_period +
-                        p_dec->i_period_remainder -
-                        ( p_dec->p_info->sequence->frame_period +
-                          p_dec->i_period_remainder ) / 27 / 2 * 27 * 2;
+                    p_dec->p_picture_to_destroy = NULL;
+                    vout_DatePicture( p_dec->p_vout, p_pic, 0 );
                 }
             }
+
+            if( p_dec->p_info->discard_fbuf &&
+                p_dec->p_info->discard_fbuf->id )
+            {
+                p_pic = (picture_t *)p_dec->p_info->discard_fbuf->id;
+                vout_UnlinkPicture( p_dec->p_vout, p_pic );
+            }
             break;
 
+        case STATE_INVALID:
+        {
+            uint8_t *buf[3];
+            buf[0] = buf[1] = buf[2] = NULL;
+
+            msg_Warn( p_dec->p_fifo, "invalid picture encountered" );
+            if ( ( p_dec->p_info->current_picture == NULL ) || 
+               ( ( p_dec->p_info->current_picture->flags & PIC_MASK_CODING_TYPE)
+                  != B_CODING_TYPE ) )
+            {
+                vout_SynchroReset( p_dec->p_synchro );
+            }
+            mpeg2_skip( p_dec->p_mpeg2dec, 1 );
+            p_dec->b_skip = 1;
+
+            if( p_dec->p_info->current_fbuf &&
+                p_dec->p_info->current_fbuf->id )
+            {
+                p_dec->b_garbage_pic = 1;
+                p_pic = p_dec->p_info->current_fbuf->id;
+            }
+            else
+            {
+                if( (p_pic = GetNewPicture( p_dec, buf )) == NULL )
+                    break;
+                mpeg2_set_buf( p_dec->p_mpeg2dec, buf, p_pic );
+            }
+            p_dec->p_picture_to_destroy = p_pic;
+
+            memset( p_pic->p[0].p_pixels, 0,
+                    p_dec->p_info->sequence->width
+                     * p_dec->p_info->sequence->height );
+            memset( p_pic->p[1].p_pixels, 0x80,
+                    p_dec->p_info->sequence->width
+                     * p_dec->p_info->sequence->height / 4 );
+            memset( p_pic->p[2].p_pixels, 0x80,
+                    p_dec->p_info->sequence->width
+                     * p_dec->p_info->sequence->height / 4 );
+
+            if ( p_dec->b_slice_i )
+            {
+                vout_SynchroNewPicture( p_dec->p_synchro,
+                            I_CODING_TYPE, 2, 0, 0, p_dec->i_current_rate );
+                vout_SynchroDecode( p_dec->p_synchro );
+                vout_SynchroEnd( p_dec->p_synchro, I_CODING_TYPE, 0 );
+            }
+            break;
+        }
+
         default:
             break;
         }
@@ -343,6 +520,9 @@ static void CloseDecoder( dec_thread_t * p_dec )
         if( p_dec->p_pes )
             input_DeletePES( p_dec->p_fifo->p_packets_mgt, p_dec->p_pes );
 
+        if( p_dec->p_synchro )
+            vout_SynchroRelease( p_dec->p_synchro );
+
         if( p_dec->p_vout )
         {
             /* Temporary hack to free the pictures in use by libmpeg2 */
@@ -352,6 +532,9 @@ static void CloseDecoder( dec_thread_t * p_dec )
                       RESERVED_PICTURE )
                     vout_DestroyPicture( p_dec->p_vout,
                                      p_dec->p_vout->render.pp_picture[i_pic] );
+                if( p_dec->p_vout->render.pp_picture[i_pic]->i_refcount > 0 )
+                    vout_UnlinkPicture( p_dec->p_vout,
+                                     p_dec->p_vout->render.pp_picture[i_pic] );
             }
 
             vout_Request( p_dec->p_fifo, p_dec->p_vout, 0, 0, 0, 0 );
@@ -369,9 +552,18 @@ static void CloseDecoder( dec_thread_t * p_dec )
 static picture_t *GetNewPicture( dec_thread_t *p_dec, uint8_t **pp_buf )
 {
     picture_t *p_pic;
+    vlc_bool_t b_progressive = p_dec->p_info->current_picture != NULL ?
+        p_dec->p_info->current_picture->flags & PIC_FLAG_PROGRESSIVE_FRAME :
+        1;
+    vlc_bool_t b_top_field_first = p_dec->p_info->current_picture != NULL ?
+        p_dec->p_info->current_picture->flags & PIC_FLAG_TOP_FIELD_FIRST :
+        1;
+    unsigned int i_nb_fields = p_dec->p_info->current_picture != NULL ?
+        p_dec->p_info->current_picture->nb_fields : 2;
 
     /* Get a new picture */
-    while( !(p_pic = vout_CreatePicture( p_dec->p_vout, 0, 0, 0 ) ) )
+    while( !(p_pic = vout_CreatePicture( p_dec->p_vout,
+        b_progressive, b_top_field_first, i_nb_fields )) )
     {
         if( p_dec->p_fifo->b_die || p_dec->p_fifo->b_error )
             break;
@@ -380,6 +572,7 @@ static picture_t *GetNewPicture( dec_thread_t *p_dec, uint8_t **pp_buf )
     }
     if( p_pic == NULL )
         return NULL;
+    vout_LinkPicture( p_dec->p_vout, p_pic );
 
     pp_buf[0] = p_pic->p[0].p_pixels;
     pp_buf[1] = p_pic->p[1].p_pixels;
