@@ -2,7 +2,7 @@
  * mp4.c : MP4 file input module for vlc
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: mp4.c,v 1.20 2003/04/06 17:48:06 gbazin Exp $
+ * $Id: mp4.c,v 1.33 2003/05/24 02:48:55 hartman Exp $
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@
 
 #include <vlc/vlc.h>
 #include <vlc/input.h>
+#include <vlc_playlist.h>
 #include "codecs.h"
 #include "libmp4.h"
 #include "mp4.h"
@@ -41,6 +42,11 @@ static int    MP4Init    ( vlc_object_t * );
 static void __MP4End     ( vlc_object_t * );
 static int    MP4Demux   ( input_thread_t * );
 
+static int    MP4DemuxRef( input_thread_t *p_input )
+{
+    return 0;
+}
+
 /* New input could have something like that... */
 static int   MP4Seek     ( input_thread_t *, mtime_t );
 
@@ -105,16 +111,19 @@ static int MP4Init( vlc_object_t * p_this )
 
     MP4_Box_t       *p_ftyp;
 
+    MP4_Box_t       *p_rmra;
 
     MP4_Box_t       *p_mvhd;
     MP4_Box_t       *p_trak;
 
     unsigned int    i;
+    vlc_bool_t      b_audio;
+
     /* I need to seek */
     if( !p_input->stream.b_seekable )
     {
         msg_Warn( p_input, "MP4 plugin discarded (unseekable)" );
-        return( -1 );
+        return( VLC_EGENERIC );
 
     }
     /* Initialize access plug-in structures. */
@@ -130,7 +139,7 @@ static int MP4Init( vlc_object_t * p_this )
     if( input_Peek( p_input, &p_peek, 8 ) < 8 )
     {
         msg_Warn( p_input, "MP4 plugin discarded (cannot peek)" );
-        return( -1 );
+        return( VLC_EGENERIC );
     }
 
 
@@ -138,6 +147,7 @@ static int MP4Init( vlc_object_t * p_this )
     {
         case( FOURCC_ftyp ):
         case( FOURCC_moov ):
+        case( FOURCC_foov ):
         case( FOURCC_moof ):
         case( FOURCC_mdat ):
         case( FOURCC_udta ):
@@ -147,7 +157,7 @@ static int MP4Init( vlc_object_t * p_this )
             break;
          default:
             msg_Warn( p_input, "MP4 plugin discarded (not a valid file)" );
-            return( -1 );
+            return( VLC_EGENERIC );
     }
 
     /* create our structure that will contains all data */
@@ -155,7 +165,7 @@ static int MP4Init( vlc_object_t * p_this )
                 p_demux = malloc( sizeof( demux_sys_t ) ) ) )
     {
         msg_Err( p_input, "out of memory" );
-        return( -1 );
+        return( VLC_EGENERIC );
     }
     memset( p_demux, 0, sizeof( demux_sys_t ) );
     p_input->p_demux_data = p_demux;
@@ -165,7 +175,7 @@ static int MP4Init( vlc_object_t * p_this )
     if( !MP4_BoxGetRoot( p_input, &p_demux->box_root ) )
     {
         msg_Warn( p_input, "MP4 plugin discarded (not a valid file)" );
-        return( -1 );
+        return( VLC_EGENERIC );
     }
 
     MP4_BoxDumpStructure( p_input, &p_demux->box_root );
@@ -192,20 +202,112 @@ static int MP4Init( vlc_object_t * p_this )
     }
 
     /* the file need to have one moov box */
-    if( MP4_BoxCount( &p_demux->box_root, "/moov" ) != 1 )
+    if( MP4_BoxCount( &p_demux->box_root, "/moov" ) <= 0 )
     {
-        msg_Err( p_input,
-                 "MP4 plugin discarded (%d moov boxes)",
-                 MP4_BoxCount( &p_demux->box_root, "/moov" ) );
-//        MP4End( p_input );
-//        return( -1 );
+        MP4_Box_t *p_foov = MP4_BoxGet( &p_demux->box_root, "/foov" );
+
+        if( !p_foov )
+        {
+            msg_Err( p_input, "MP4 plugin discarded (no moov box)" );
+            MP4End( p_input );
+            return( VLC_EGENERIC );
+        }
+        /* we have a free box as a moov, rename it */
+        p_foov->i_type = FOURCC_moov;
+    }
+
+    if( ( p_rmra = MP4_BoxGet( &p_demux->box_root,  "/moov/rmra" ) ) )
+    {
+        playlist_t *p_playlist;
+        int        i_count = MP4_BoxCount( p_rmra, "rmda" );
+        int        i;
+
+        msg_Dbg( p_input, "detected playlist mov file (%d ref)", i_count );
+
+        p_playlist =
+            (playlist_t *)vlc_object_find( p_input,
+                                           VLC_OBJECT_PLAYLIST,
+                                           FIND_ANYWHERE );
+        if( p_playlist )
+        {
+            //p_playlist->pp_items[p_playlist->i_index]->b_autodeletion = VLC_TRUE;
+            for( i = 0; i < i_count; i++ )
+            {
+                MP4_Box_t *p_rdrf = MP4_BoxGet( p_rmra, "rmda[%d]/rdrf", i );
+                char      *psz_ref;
+                uint32_t  i_ref_type;
+
+                if( !p_rdrf || !( psz_ref = p_rdrf->data.p_rdrf->psz_ref ) )
+                {
+                    continue;
+                }
+                i_ref_type = p_rdrf->data.p_rdrf->i_ref_type;
+
+                msg_Dbg( p_input, "new ref=`%s' type=%4.4s",
+                         psz_ref, (char*)&i_ref_type );
+
+                if( i_ref_type == VLC_FOURCC( 'u', 'r', 'l', ' ' ) )
+                {
+                    if( strstr( psz_ref, "qt5gateQT" ) )
+                    {
+                        msg_Dbg( p_input, "ignoring pseudo ref =`%s'", psz_ref );
+                        continue;
+                    }
+                    if( !strncmp( psz_ref, "http://", 7 ) ||
+                        !strncmp( psz_ref, "rtsp://", 7 ) )
+                    {
+                        msg_Dbg( p_input, "adding ref = `%s'", psz_ref );
+                        playlist_Add( p_playlist, psz_ref, PLAYLIST_APPEND, PLAYLIST_END );
+                    }
+                    else
+                    {
+                        /* msg dbg relative ? */
+                        char *psz_absolute = alloca( strlen( p_input->psz_source ) + strlen( psz_ref ) + 1);
+                        char *end = strrchr( p_input->psz_source, '/' );
+
+                        if( end )
+                        {
+                            int i_len = end + 1 - p_input->psz_source;
+
+                            strncpy( psz_absolute, p_input->psz_source, i_len);
+                            psz_absolute[i_len] = '\0';
+                        }
+                        else
+                        {
+                            strcpy( psz_absolute, "" );
+                        }
+                        strcat( psz_absolute, psz_ref );
+                        msg_Dbg( p_input, "adding ref = `%s'", psz_absolute );
+                        playlist_Add( p_playlist, psz_absolute, PLAYLIST_APPEND, PLAYLIST_END );
+                    }
+                }
+                else
+                {
+                    msg_Err( p_input, "unknown ref type=%4.4s FIXME (send a bug report)", (char*)&p_rdrf->data.p_rdrf->i_ref_type );
+                }
+            }
+            vlc_object_release( p_playlist );
+        }
+        else
+        {
+            msg_Err( p_input, "can't find playlist" );
+        }
     }
 
     if( !(p_mvhd = MP4_BoxGet( &p_demux->box_root, "/moov/mvhd" ) ) )
     {
-        msg_Err( p_input, "cannot find /moov/mvhd" );
-        MP4End( p_input );
-        return( -1 );
+        if( !p_rmra )
+        {
+            msg_Err( p_input, "cannot find /moov/mvhd" );
+            MP4End( p_input );
+            return VLC_EGENERIC;
+        }
+        else
+        {
+            msg_Warn( p_input, "cannot find /moov/mvhd (pure ref file)" );
+            p_input->pf_demux = MP4DemuxRef;
+            return VLC_SUCCESS;
+        }
     }
     else
     {
@@ -218,7 +320,7 @@ static int MP4Init( vlc_object_t * p_this )
     {
         msg_Err( p_input, "cannot find any /moov/trak" );
         MP4End( p_input );
-        return( -1 );
+        return( VLC_EGENERIC );
     }
     msg_Dbg( p_input, "find %d track%c",
                         p_demux->i_tracks,
@@ -231,7 +333,7 @@ static int MP4Init( vlc_object_t * p_this )
         vlc_mutex_unlock( &p_input->stream.stream_lock );
         msg_Err( p_input, "cannot init stream" );
         MP4End( p_input );
-        return( -1 );
+        return( VLC_EGENERIC );
     }
     /* Needed to create program _before_ MP4_TrackCreate */
     if( input_AddProgram( p_input, 0, 0) == NULL )
@@ -239,7 +341,7 @@ static int MP4Init( vlc_object_t * p_this )
         vlc_mutex_unlock( &p_input->stream.stream_lock );
         msg_Err( p_input, "cannot add program" );
         MP4End( p_input );
-        return( -1 );
+        return( VLC_EGENERIC );
     }
     p_input->stream.p_selected_program = p_input->stream.pp_programs[0];
     /* XXX beurk and beurk, see MP4Demux and MP4Seek */
@@ -296,21 +398,29 @@ static int MP4Init( vlc_object_t * p_this )
 
     }
 
-    for( i = 0; i < p_demux->i_tracks; i++ )
+    for( i = 0, b_audio = VLC_FALSE; i < p_demux->i_tracks; i++ )
     {
+#define track p_demux->track[i]
         /* start decoder for this track if enable by default*/
-        if( p_demux->track[i].b_ok && p_demux->track[i].b_enable )
+        if( track.b_ok && track.b_enable &&
+            ( track.i_cat != AUDIO_ES || !b_audio ) )
         {
-            MP4_TrackSelect( p_input, &p_demux->track[i], 0 );
+            if( !MP4_TrackSelect( p_input, &track, 0 ) )
+            {
+                if(track.i_cat == AUDIO_ES )
+                {
+                    b_audio = VLC_TRUE;
+                }
+            }
         }
+#undef track
     }
 
     vlc_mutex_lock( &p_input->stream.stream_lock );
     p_input->stream.p_selected_program->b_is_ok = 1;
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
-    return( 0 );
-
+    return( VLC_SUCCESS );
 }
 
 /*****************************************************************************
@@ -401,17 +511,8 @@ static int MP4Demux( input_thread_t *p_input )
     p_demux->i_time += __MAX( p_demux->i_timescale / 10 , 1 );
 
 
-    /* *** send audio data to decoder if rate == DEFAULT_RATE or no video *** */
-    vlc_mutex_lock( &p_input->stream.stream_lock );
-    if( p_input->stream.control.i_rate == DEFAULT_RATE || !b_video )
-    {
-        b_play_audio = VLC_TRUE;
-    }
-    else
-    {
-        b_play_audio = VLC_FALSE;
-    }
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
+    /* Check if we need to send the audio data to decoder */
+    b_play_audio = !p_input->stream.control.b_mute;
 
     for( i_track = 0; i_track < p_demux->i_tracks; i_track++ )
     {
@@ -445,8 +546,10 @@ static int MP4Demux( input_thread_t *p_input )
 
                 i_pos  = MP4_GetTrackPos( &track );
 
+                //msg_Dbg( p_input, "stream %d size=%6d pos=%8lld",  i_track, i_size, i_pos );
+
                 /* go,go go ! */
-                if( ! MP4_SeekAbsolute( p_input, i_pos ) )
+                if( MP4_SeekAbsolute( p_input, i_pos ) )
                 {
                     msg_Warn( p_input, "track[0x%x] will be disabled (eof?)", track.i_track_ID );
                     MP4_TrackUnselect( p_input, &track );
@@ -465,6 +568,8 @@ static int MP4Demux( input_thread_t *p_input )
                     input_DeletePES( p_input->p_method_data, p_pes );
                     break;
                 }
+                p_data->p_payload_end = p_data->p_payload_start + i_size;
+
                 /* initialisation of all the field */
                 p_pes->i_dts = p_pes->i_pts = 0;
                 p_pes->p_first = p_pes->p_last  = p_data;
@@ -472,7 +577,7 @@ static int MP4Demux( input_thread_t *p_input )
                 p_pes->i_pes_size = i_size;
                 if( i_size > 0 )
                 {
-                    if( !MP4_ReadData( p_input, p_data->p_payload_start, i_size ) )
+                    if( MP4_ReadData( p_input, p_data->p_payload_start, i_size ) )
                     {
                         input_DeletePES( p_input->p_method_data, p_pes );
 
@@ -489,6 +594,8 @@ static int MP4Demux( input_thread_t *p_input )
 
                 if( track.p_es->p_decoder_fifo )
                 {
+
+                    p_pes->i_rate = p_input->stream.control.i_rate;
                     input_DecodePES( track.p_es->p_decoder_fifo, p_pes );
                 }
                 else
@@ -774,6 +881,7 @@ static int  TrackCreateES   ( input_thread_t   *p_input,
 {
     MP4_Box_t *  p_sample;
     unsigned int i;
+    char psz_lang[4];
 
     unsigned int i_decoder_specific_info_len;
     uint8_t *    p_decoder_specific_info;
@@ -826,25 +934,37 @@ static int  TrackCreateES   ( input_thread_t   *p_input,
                     p_soun->i_bytes_per_frame   = 34 * p_soun->i_channelcount;
                     p_soun->i_bytes_per_sample  = 2;
                     break;
+                case VLC_FOURCC( 'M', 'A', 'C', '3' ):
+                    p_soun->i_qt_version = 1;
+                    p_soun->i_sample_per_packet = 6;
+                    p_soun->i_bytes_per_packet  = 2;
+                    p_soun->i_bytes_per_frame   = 2 * p_soun->i_channelcount;
+                    p_soun->i_bytes_per_sample  = 2;
+                    break;
+                case VLC_FOURCC( 'M', 'A', 'C', '6' ):
+                    p_soun->i_qt_version = 1;
+                    p_soun->i_sample_per_packet = 12;
+                    p_soun->i_bytes_per_packet  = 2;
+                    p_soun->i_bytes_per_frame   = 2 * p_soun->i_channelcount;
+                    p_soun->i_bytes_per_sample  = 2;
+                    break;
                 default:
                     break;
             }
         }
     }
 
-
-    vlc_mutex_lock( &p_input->stream.stream_lock );
-    p_es = input_AddES( p_input,
-                        p_input->stream.p_selected_program,
-                        p_track->i_track_ID,
-                        0 );
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
     /* Initialise ES, first language as description */
     for( i = 0; i < 3; i++ )
     {
-        p_es->psz_desc[i] = p_track->i_language[i];
+        psz_lang[i] = p_track->i_language[i];
     }
-    p_es->psz_desc[3] = '\0';
+    psz_lang[3] = '\0';
+
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+    p_es = input_AddES( p_input, p_input->stream.p_selected_program,
+                        p_track->i_track_ID, p_track->i_cat, psz_lang, 0 );
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
 
     p_es->i_stream_id = p_track->i_track_ID;
 
@@ -858,13 +978,14 @@ static int  TrackCreateES   ( input_thread_t   *p_input,
         case( VLC_FOURCC( 'r', 'a', 'w', ' ' ) ):
             p_es->i_fourcc = VLC_FOURCC( 'a', 'r', 'a', 'w' );
             break;
+        case( VLC_FOURCC( 's', '2', '6', '3' ) ):
+            p_es->i_fourcc = VLC_FOURCC( 'h', '2', '6', '3' );
+            break;
         default:
             p_es->i_fourcc = p_sample->i_type;
             break;
     }
 
-    p_es->i_cat = p_track->i_cat;
-
     i_decoder_specific_info_len = 0;
     p_decoder_specific_info = NULL;
     p_pes_init = NULL;
@@ -925,6 +1046,29 @@ static int  TrackCreateES   ( input_thread_t   *p_input,
         p_decoder_specific_info =
                 p_decconfig->p_decoder_specific_info;
     }
+    else
+    {
+        switch( p_sample->i_type )
+        {
+            /* qt decoder, send the complete chunk */
+            case VLC_FOURCC( 'S', 'V', 'Q', '3' ):
+            case VLC_FOURCC( 'S', 'V', 'Q', '1' ):
+            case VLC_FOURCC( 'V', 'P', '3', '1' ):
+            case VLC_FOURCC( '3', 'I', 'V', '1' ):
+            case VLC_FOURCC( 'Z', 'y', 'G', 'o' ):
+                i_decoder_specific_info_len = p_sample->data.p_sample_vide->i_qt_image_description;
+                p_decoder_specific_info     = p_sample->data.p_sample_vide->p_qt_image_description;
+                break;
+            case VLC_FOURCC( 'Q', 'D', 'M', 'C' ):
+            case VLC_FOURCC( 'Q', 'D', 'M', '2' ):
+            case VLC_FOURCC( 'Q', 'c', 'l', 'p' ):
+                i_decoder_specific_info_len = p_sample->data.p_sample_soun->i_qt_description;
+                p_decoder_specific_info     = p_sample->data.p_sample_soun->p_qt_description;
+                break;
+            default:
+                break;
+        }
+    }
 
 #undef p_decconfig
 
@@ -993,6 +1137,8 @@ static int  TrackCreateES   ( input_thread_t   *p_input,
                         p_pes_init = input_NewPES( p_input->p_method_data );
                         p_data = input_NewPacket( p_input->p_method_data,
                                                   i_decoder_specific_info_len);
+                        p_data->p_payload_end = p_data->p_payload_start + i_decoder_specific_info_len;
+
                         memcpy( p_data->p_payload_start,
                                 p_decoder_specific_info,
                                 i_decoder_specific_info_len );
@@ -1230,7 +1376,10 @@ static int  TrackGotoChunkSample( input_thread_t   *p_input,
         {
             if( p_track->p_pes_init != NULL )
             {
-                input_DecodePES( p_track->p_es->p_decoder_fifo, p_track->p_pes_init );
+                p_track->p_pes_init->i_rate = p_input->stream.control.i_rate;
+
+                input_DecodePES( p_track->p_es->p_decoder_fifo,
+                                 p_track->p_pes_init );
                 p_track->p_pes_init = NULL;
             }
             p_track->b_selected = VLC_TRUE;
@@ -1250,7 +1399,7 @@ static int  TrackGotoChunkSample( input_thread_t   *p_input,
     p_track->i_chunk    = i_chunk;
     p_track->i_sample   = i_sample;
 
-    return( VLC_SUCCESS );
+    return( p_track->b_selected ? VLC_SUCCESS : VLC_EGENERIC );
 }
 
 /****************************************************************************
@@ -1358,6 +1507,24 @@ static void MP4_TrackCreate( input_thread_t *p_input,
         return;
     }
 
+    /* fxi i_timescale for AUDIO_ES with i_qt_version == 0 */
+    if( p_track->i_cat == AUDIO_ES ) //&& p_track->i_sample_size == 1 )
+    {
+        MP4_Box_t *p_sample;
+
+        p_sample = MP4_BoxGet(  p_track->p_stsd, "[0]" );
+        if( p_sample && p_sample->data.p_sample_soun)
+        {
+            MP4_Box_data_sample_soun_t *p_soun = p_sample->data.p_sample_soun;
+            if( p_soun->i_qt_version == 0 && p_track->i_timescale != p_soun->i_sampleratehi )
+            {
+                msg_Warn( p_input, "i_timescale != i_sampleratehi with qt_version == 0\nMaking both equal ? (report any problem)" );
+                p_track->i_timescale = p_soun->i_sampleratehi;
+            }
+        }
+    }
+
+
     /* Create chunk  index table */
     if( TrackCreateChunksIndex( p_input,p_track  ) )
     {
@@ -1537,7 +1704,7 @@ static int  MP4_TrackSeek   ( input_thread_t    *p_input,
  * 3 types: for audio
  * 
  */
-
+#define QT_V0_MAX_SAMPLES    1500
 static int  MP4_TrackSampleSize( track_data_mp4_t   *p_track )
 {
     int i_size;
@@ -1567,10 +1734,14 @@ static int  MP4_TrackSampleSize( track_data_mp4_t   *p_track )
     }
     else
     {
-        //msg_Warn( p_input, "i_qt_version == 0" );
         /* FIXME */
-
-        i_size = p_track->chunk[p_track->i_chunk].i_sample_count * p_soun->i_channelcount * p_soun->i_samplesize / 8;
+        int i_samples = p_track->chunk[p_track->i_chunk].i_sample_count -
+                ( p_track->i_sample - p_track->chunk[p_track->i_chunk].i_sample_first );
+        if( i_samples > QT_V0_MAX_SAMPLES )
+        {
+            i_samples = QT_V0_MAX_SAMPLES;
+        }
+        i_size = i_samples * p_soun->i_channelcount * p_soun->i_samplesize / 8;
     }
 
     //fprintf( stderr, "size=%d\n", i_size );
@@ -1588,13 +1759,18 @@ static uint64_t MP4_GetTrackPos( track_data_mp4_t *p_track )
 
     if( p_track->i_sample_size )
     {
-#if 0
-        i_pos += ( p_track->i_sample -
-                        p_track->chunk[p_track->i_chunk].i_sample_first ) *
-                                MP4_TrackSampleSize( p_track );
-#endif
-        /* we read chunk by chunk */
-        i_pos += 0;
+        MP4_Box_data_sample_soun_t *p_soun = p_track->p_sample->data.p_sample_soun;
+
+        if( p_soun->i_qt_version == 0 )
+        {
+            i_pos += ( p_track->i_sample - p_track->chunk[p_track->i_chunk].i_sample_first ) *
+                        p_soun->i_channelcount * p_soun->i_samplesize / 8;
+        }
+        else
+        {
+            /* we read chunk by chunk */
+            i_pos += 0;
+        }
     }
     else
     {
@@ -1629,9 +1805,13 @@ static int  MP4_TrackNextSample( input_thread_t     *p_input,
         else
         {
             /* FIXME */
-            p_track->i_sample =
-                p_track->chunk[p_track->i_chunk].i_sample_first +
-                p_track->chunk[p_track->i_chunk].i_sample_count;
+            p_track->i_sample += QT_V0_MAX_SAMPLES;
+            if( p_track->i_sample > p_track->chunk[p_track->i_chunk].i_sample_first + p_track->chunk[p_track->i_chunk].i_sample_count )
+            {
+                p_track->i_sample =
+                    p_track->chunk[p_track->i_chunk].i_sample_first +
+                    p_track->chunk[p_track->i_chunk].i_sample_count;
+            }
         }
     }
     else
