@@ -2,7 +2,7 @@
  * intf.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: intf.m,v 1.69 2003/04/06 23:21:13 massiot Exp $
+ * $Id: intf.m,v 1.90 2003/06/17 14:43:22 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -35,6 +35,7 @@
 #include "prefs.h"
 #include "playlist.h"
 #include "info.h"
+#include "controls.h"
 
 /*****************************************************************************
  * Local prototypes.
@@ -55,14 +56,19 @@ int E_(OpenIntf) ( vlc_object_t *p_this )
     }
 
     memset( p_intf->p_sys, 0, sizeof( *p_intf->p_sys ) );
-
+    
     p_intf->p_sys->o_pool = [[NSAutoreleasePool alloc] init];
+    
+    /* Put Cocoa into multithread mode as soon as possible.
+     * http://developer.apple.com/techpubs/macosx/Cocoa/
+     * TasksAndConcepts/ProgrammingTopics/Multithreading/index.html
+     * This thread does absolutely nothing at all. */
+    [NSThread detachNewThreadSelector:@selector(self) toTarget:[NSString string] withObject:nil];
+    
     p_intf->p_sys->o_sendport = [[NSPort port] retain];
-
     p_intf->p_sys->p_sub = msg_Subscribe( p_intf );
-
     p_intf->pf_run = Run;
-
+    
     [[VLCApplication sharedApplication] autorelease];
     [NSApp initIntlSupport];
     [NSApp setIntf: p_intf];
@@ -81,8 +87,6 @@ void E_(CloseIntf) ( vlc_object_t *p_this )
 
     msg_Unsubscribe( p_intf, p_intf->p_sys->p_sub );
 
-    config_SaveConfigFile( p_intf, MODULE_STRING );
-
     [p_intf->p_sys->o_sendport release];
     [p_intf->p_sys->o_pool release];
 
@@ -110,34 +114,13 @@ static void Run( intf_thread_t *p_intf )
 - (id)init
 {
     /* default encoding: ISO-8859-1 */
-    i_encoding = NSISOLatin1StringEncoding;
+    i_encoding = NSUTF8StringEncoding;
 
     return( [super init] );
 }
 
 - (void)initIntlSupport
 {
-    char *psz_lang = getenv( "LANG" );
-
-    if( psz_lang == NULL )
-    {
-        return;
-    }
-
-    if( strncmp( psz_lang, "pl", 2 ) == 0 )
-    {
-        i_encoding = NSISOLatin2StringEncoding;
-    }
-    else if( strncmp( psz_lang, "ja", 2 ) == 0 ) 
-    {
-        i_encoding = NSJapaneseEUCStringEncoding;
-    }
-    else if( strncmp( psz_lang, "ru", 2 ) == 0 )
-    {
-#define CFSENC2NSSENC(e) CFStringConvertEncodingToNSStringEncoding(e)
-        i_encoding = CFSENC2NSSENC( kCFStringEncodingKOI8_R ); 
-#undef CFSENC2NSSENC
-    }
 }
 
 - (NSString *)localizedString:(char *)psz
@@ -146,10 +129,11 @@ static void Run( intf_thread_t *p_intf )
 
     if( psz != NULL )
     {
-        UInt32 uiLength = (UInt32)strlen( psz );
-        NSData * o_data = [NSData dataWithBytes: psz length: uiLength];
-        o_str = [[[NSString alloc] initWithData: o_data
-                                       encoding: i_encoding] autorelease];
+        o_str = [[[NSString alloc] initWithUTF8String: psz] autorelease];
+    }
+    if ( o_str == NULL )
+    {
+        msg_Err( p_intf, "could not translate: %s", psz );
     }
 
     return( o_str );
@@ -186,6 +170,49 @@ static void Run( intf_thread_t *p_intf )
     return i_encoding;
 }
 
+/* i_width is in pixels */
+- (NSString *)wrapString: (NSString *)o_in_string toWidth: (int) i_width
+{
+    NSMutableString *o_wrapped;
+    NSString *o_out_string;
+    NSRange glyphRange, effectiveRange, charRange;
+    NSRect lineFragmentRect;
+    unsigned glyphIndex, breaksInserted = 0;
+
+    NSTextStorage *o_storage = [[NSTextStorage alloc] initWithString: o_in_string
+        attributes: [NSDictionary dictionaryWithObjectsAndKeys:
+        [NSFont labelFontOfSize: 0.0], NSFontAttributeName, nil]];
+    NSLayoutManager *o_layout_manager = [[NSLayoutManager alloc] init];
+    NSTextContainer *o_container = [[NSTextContainer alloc]
+        initWithContainerSize: NSMakeSize(i_width, 2000)];
+    
+    [o_layout_manager addTextContainer: o_container];
+    [o_container release];
+    [o_storage addLayoutManager: o_layout_manager];
+    [o_layout_manager release];
+        
+    o_wrapped = [o_in_string mutableCopy];
+    glyphRange = [o_layout_manager glyphRangeForTextContainer: o_container];
+    
+    for( glyphIndex = glyphRange.location ; glyphIndex < NSMaxRange(glyphRange) ;
+            glyphIndex += effectiveRange.length) {
+        lineFragmentRect = [o_layout_manager lineFragmentRectForGlyphAtIndex: glyphIndex
+                                            effectiveRange: &effectiveRange];
+        charRange = [o_layout_manager characterRangeForGlyphRange: effectiveRange
+                                    actualGlyphRange: &effectiveRange];
+        if ([o_wrapped lineRangeForRange:
+                NSMakeRange(charRange.location + breaksInserted, charRange.length)].length > charRange.length) {
+            [o_wrapped insertString: @"\n" atIndex: NSMaxRange(charRange) + breaksInserted];
+            breaksInserted++;
+        }
+    }
+    o_out_string = [NSString stringWithString: o_wrapped];
+    [o_wrapped release];
+    [o_storage release];
+    
+    return o_out_string;
+}
+
 - (void)setIntf:(intf_thread_t *)_p_intf
 {
     p_intf = _p_intf;
@@ -267,6 +294,19 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     return( i_ret );
 }
 
+/*****************************************************************************
+ * playlistChanged: Callback triggered by the intf-change playlist
+ * variable, to let the intf update the playlist.
+ *****************************************************************************/
+int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
+                     vlc_value_t old_val, vlc_value_t new_val, void *param )
+{
+    intf_thread_t * p_intf = [NSApp getIntf];
+    p_intf->p_sys->b_playlist_update = VLC_TRUE;
+    p_intf->p_sys->b_intf_update = VLC_TRUE;
+    return VLC_SUCCESS;
+}
+
 /*****************************************************************************
  * VLCMain implementation 
  *****************************************************************************/
@@ -329,24 +369,36 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     [o_mi_fwd setTitle: _NS("Step Forward")];
     [o_mi_bwd setTitle: _NS("Step Backward")];
     [o_mi_program setTitle: _NS("Program")];
+    [o_mu_program setTitle: _NS("Program")];
     [o_mi_title setTitle: _NS("Title")];
+    [o_mu_title setTitle: _NS("Title")];
     [o_mi_chapter setTitle: _NS("Chapter")];
-    [o_mi_language setTitle: _NS("Language")];
-    [o_mi_subtitle setTitle: _NS("Subtitles")];
+    [o_mu_chapter setTitle: _NS("Chapter")];
     
     [o_mu_audio setTitle: _NS("Audio")];
     [o_mi_vol_up setTitle: _NS("Volume Up")];
     [o_mi_vol_down setTitle: _NS("Volume Down")];
     [o_mi_mute setTitle: _NS("Mute")];
-    [o_mi_channels setTitle: _NS("Channels")];
-    [o_mi_device setTitle: _NS("Device")];
+    [o_mi_audiotrack setTitle: _NS("Audio track")];
+    [o_mu_audiotrack setTitle: _NS("Audio track")];
+    [o_mi_channels setTitle: _NS("Audio channels")];
+    [o_mu_channels setTitle: _NS("Audio channels")];
+    [o_mi_device setTitle: _NS("Audio device")];
+    [o_mu_device setTitle: _NS("Audio device")];
     
     [o_mu_video setTitle: _NS("Video")];
     [o_mi_half_window setTitle: _NS("Half Size")];
     [o_mi_normal_window setTitle: _NS("Normal Size")];
     [o_mi_double_window setTitle: _NS("Double Size")];
+    [o_mi_fittoscreen setTitle: _NS("Fit To Screen")];
     [o_mi_fullscreen setTitle: _NS("Fullscreen")];
-    [o_mi_screen setTitle: _NS("Screen")];
+    [o_mi_floatontop setTitle: _NS("Float On Top")];
+    [o_mi_videotrack setTitle: _NS("Video track")];
+    [o_mu_videotrack setTitle: _NS("Video track")];
+    [o_mi_screen setTitle: _NS("Video device")];
+    [o_mu_screen setTitle: _NS("Video device")];
+    [o_mi_subtitle setTitle: _NS("Subtitles track")];
+    [o_mu_subtitle setTitle: _NS("Subtitles track")];
     [o_mi_deinterlace setTitle: _NS("Deinterlace")];
     [o_mu_deinterlace setTitle: _NS("Deinterlace")];
 
@@ -362,6 +414,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
     [o_mu_help setTitle: _NS("Help")];
     [o_mi_readme setTitle: _NS("ReadMe...")];
+    [o_mi_documentation setTitle: _NS("Online Documentation")];
     [o_mi_reportabug setTitle: _NS("Report a Bug")];
     [o_mi_website setTitle: _NS("VideoLAN Website")];
     [o_mi_license setTitle: _NS("License")];
@@ -412,9 +465,16 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
 - (BOOL)application:(NSApplication *)o_app openFile:(NSString *)o_filename
 {
+    intf_thread_t * p_intf = [NSApp getIntf];
+    
+    config_PutPsz( p_intf, "sub-file", "" );
+    config_PutInt( p_intf, "sub-delay", 0 );
+    config_PutFloat( p_intf, "sub-fps", 0.0 );
+    config_PutPsz( p_intf, "sout", "" );
+    
     [o_playlist appendArray:
         [NSArray arrayWithObject: o_filename] atPos: -1 enqueue: NO];
-
+            
     return( TRUE );
 }
 
@@ -446,6 +506,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
         if( p_playlist != NULL )
         {
+            var_AddCallback( p_playlist, "intf-change", PlaylistChanged, self );
             vlc_mutex_lock( &p_playlist->object_lock );
             
             [self manage: p_playlist];
@@ -467,92 +528,36 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
 - (void)manage:(playlist_t *)p_playlist
 {
-    vlc_value_t val;
-
     intf_thread_t * p_intf = [NSApp getIntf];
 
-    if( var_Get( (vlc_object_t *)p_playlist, "intf-change", &val ) >= 0 &&
-        val.b_bool )
-    {
-        p_intf->p_sys->b_playlist_update = VLC_TRUE;
-        p_intf->p_sys->b_intf_update = VLC_TRUE;
-    }
-
 #define p_input p_playlist->p_input
 
     if( p_input )
     {
-        vout_thread_t   * p_vout  = NULL;
-        aout_instance_t * p_aout  = NULL; 
-        vlc_bool_t b_need_menus = VLC_FALSE;
-
         vlc_mutex_lock( &p_input->stream.stream_lock );
 
         if( !p_input->b_die )
         {
-            audio_volume_t i_volume;
-
             /* New input or stream map change */
             if( p_input->stream.b_changed )
             {
-                p_intf->p_sys->b_playing = 1;
+                p_intf->p_sys->b_playing = TRUE;
                 [self manageMode: p_playlist];
-                b_need_menus = VLC_TRUE;
             }
 
-            if( p_intf->p_sys->i_part !=
-                p_input->stream.p_selected_area->i_part )
+            vlc_value_t val;
+
+            if( var_Get( (vlc_object_t *)p_input, "intf-change", &val )
+                >= 0 && val.b_bool )
             {
-                p_intf->p_sys->b_chapter_update = 1;
-                b_need_menus = VLC_TRUE;
-            }
-
-            p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                              FIND_ANYWHERE );
-            if( p_aout != NULL )
-            {
-                vlc_value_t val;
-
-                if( var_Get( (vlc_object_t *)p_aout, "intf-change", &val )
-                    >= 0 && val.b_bool )
-                {
-                    p_intf->p_sys->b_aout_update = 1;
-                    b_need_menus = VLC_TRUE;
-                }
-
-                vlc_object_release( (vlc_object_t *)p_aout );
-            }
-
-            aout_VolumeGet( p_intf, &i_volume );
-            p_intf->p_sys->b_mute = ( i_volume == 0 );
-
-            p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                              FIND_ANYWHERE );
-            if( p_vout != NULL )
-            {
-                vlc_value_t val;
-
-                if( var_Get( (vlc_object_t *)p_vout, "intf-change", &val )
-                    >= 0 && val.b_bool )
-                {
-                    p_intf->p_sys->b_vout_update = 1;
-                    b_need_menus = VLC_TRUE;
-                }
-
-                vlc_object_release( (vlc_object_t *)p_vout );
-            } 
-
-            if( b_need_menus )
-            {
-                [self setupMenus: p_input];
+                p_intf->p_sys->b_input_update = TRUE;
             }
         }
-
         vlc_mutex_unlock( &p_input->stream.stream_lock );
     }
     else if( p_intf->p_sys->b_playing && !p_intf->b_die )
     {
-        p_intf->p_sys->b_playing = 0;
+        p_intf->p_sys->b_playing = FALSE;
         [self manageMode: p_playlist];
     }
 
@@ -565,15 +570,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
     if( p_playlist->p_input != NULL )
     {
-        /* get ready for menu regeneration */
-        p_intf->p_sys->b_program_update = 1;
-        p_intf->p_sys->b_title_update = 1;
-        p_intf->p_sys->b_chapter_update = 1;
-        p_intf->p_sys->b_audio_update = 1;
-        p_intf->p_sys->b_spu_update = 1;
         p_intf->p_sys->b_current_title_update = 1;
-        p_intf->p_sys->i_part = 0;
-
         p_playlist->p_input->stream.b_changed = 0;
         
         msg_Dbg( p_intf, "stream has changed, refreshing interface" );
@@ -616,30 +613,29 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
     if ( p_intf->p_sys->b_playlist_update )
     {
-        vlc_value_t val;
-
-        val.b_bool = 0;
-        var_Set( (vlc_object_t *)p_playlist, "intf-change", val );
-
         [o_playlist playlistUpdated];
-
         p_intf->p_sys->b_playlist_update = VLC_FALSE;
     }
 
     if( p_intf->p_sys->b_current_title_update )
     {
-        id o_vout_wnd;
-
-        NSEnumerator * o_enum = [[NSApp windows] objectEnumerator];
-        
-        while( ( o_vout_wnd = [o_enum nextObject] ) )
+        vout_thread_t   *p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
+                                              FIND_ANYWHERE );
+        if( p_vout != NULL )
         {
-            if( [[o_vout_wnd className] isEqualToString: @"VLCWindow"] )
+            /* We need to lock the vout here to make sure it does not disappear */
+            id o_vout_wnd;
+            NSEnumerator * o_enum = [[NSApp windows] objectEnumerator];
+            
+            while( ( o_vout_wnd = [o_enum nextObject] ) )
             {
-                [o_vout_wnd updateTitle];
+                if( [[o_vout_wnd className] isEqualToString: @"VLCWindow"] )
+                {
+                    [o_vout_wnd updateTitle];
+                }
             }
+            vlc_object_release( (vlc_object_t *)p_vout );
         }
-
         [o_playlist updateRowSelection];
         [o_info updateInfo];
 
@@ -683,8 +679,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
         else
         {
             /* play status */
-            p_intf->p_sys->b_play_status = VLC_FALSE;
-
+            p_intf->p_sys->b_play_status = FALSE;
             [self setSubmenusEnabled: FALSE];
         }
 
@@ -709,7 +704,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
     if( p_intf->p_sys->b_playing && p_input != NULL )
     {
-        vlc_bool_t b_field_update = VLC_TRUE;
+        vlc_bool_t b_field_update = TRUE;
 
         if( !p_input->b_die && ( p_intf->p_sys->b_play_status !=
             ( p_input->stream.control.i_status != PAUSE_S ) ) ) 
@@ -755,7 +750,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
             input_OffsetToTime( p_input, psz_time, p_area->i_tell );
 
-            o_time = [NSString stringWithCString: psz_time];
+            o_time = [NSString stringWithUTF8String: psz_time];
             [o_timefield setStringValue: o_time];
         }
 
@@ -782,6 +777,65 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
         userInfo: nil repeats: FALSE];
 }
 
+- (void)setupMenus
+{
+    intf_thread_t * p_intf = [NSApp getIntf];
+    playlist_t *p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST, 
+                                                FIND_ANYWHERE );
+    
+    if( p_playlist != NULL )
+    {
+#define p_input p_playlist->p_input
+        if( p_input != NULL )
+        {
+            [o_controls setupVarMenuItem: o_mi_program target: (vlc_object_t *)p_input
+                var: "program" selector: @selector(toggleVar:)];
+            
+            [o_controls setupVarMenuItem: o_mi_title target: (vlc_object_t *)p_input
+                var: "title" selector: @selector(toggleVar:)];
+            
+            [o_controls setupVarMenuItem: o_mi_chapter target: (vlc_object_t *)p_input
+                var: "chapter" selector: @selector(toggleVar:)];
+            
+            [o_controls setupVarMenuItem: o_mi_audiotrack target: (vlc_object_t *)p_input
+                var: "audio-es" selector: @selector(toggleVar:)];
+            
+            [o_controls setupVarMenuItem: o_mi_videotrack target: (vlc_object_t *)p_input
+                var: "video-es" selector: @selector(toggleVar:)];
+            
+            [o_controls setupVarMenuItem: o_mi_subtitle target: (vlc_object_t *)p_input
+                var: "spu-es" selector: @selector(toggleVar:)];
+        
+            aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
+                                                        FIND_ANYWHERE );
+            if ( p_aout != NULL )
+            {
+                [o_controls setupVarMenuItem: o_mi_channels target: (vlc_object_t *)p_aout
+                    var: "audio-channels" selector: @selector(toggleVar:)];
+            
+                [o_controls setupVarMenuItem: o_mi_device target: (vlc_object_t *)p_aout
+                    var: "audio-device" selector: @selector(toggleVar:)];
+                vlc_object_release( (vlc_object_t *)p_aout );
+            }
+            
+            vout_thread_t * p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
+                                                                FIND_ANYWHERE );
+            
+            if ( p_vout != NULL )
+            {
+                [o_controls setupVarMenuItem: o_mi_screen target: (vlc_object_t *)p_vout
+                    var: "video-device" selector: @selector(toggleVar:)];
+                
+                [o_controls setupVarMenuItem: o_mi_deinterlace target: (vlc_object_t *)p_vout
+                    var: "deinterlace" selector: @selector(toggleVar:)];
+                vlc_object_release( (vlc_object_t *)p_vout );
+            }
+        }
+#undef p_input
+    }
+    vlc_object_release( (vlc_object_t *)p_playlist );
+}
+
 - (void)updateMessageArray
 {
     int i_start, i_stop;
@@ -885,9 +939,11 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     [o_mi_program setEnabled: b_enabled];
     [o_mi_title setEnabled: b_enabled];
     [o_mi_chapter setEnabled: b_enabled];
-    [o_mi_language setEnabled: b_enabled];
+    [o_mi_audiotrack setEnabled: b_enabled];
+    [o_mi_videotrack setEnabled: b_enabled];
     [o_mi_subtitle setEnabled: b_enabled];
     [o_mi_channels setEnabled: b_enabled];
+    [o_mi_deinterlace setEnabled: b_enabled];
     [o_mi_device setEnabled: b_enabled];
     [o_mi_screen setEnabled: b_enabled];
 }
@@ -900,11 +956,63 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     aout_VolumeGet( p_intf, &i_volume );
 
     [o_volumeslider setFloatValue: (float)i_volume / AOUT_VOLUME_STEP]; 
-    [o_volumeslider setEnabled: 1];
+    [o_volumeslider setEnabled: TRUE];
 
     p_intf->p_sys->b_mute = ( i_volume == 0 );
 }
 
+- (IBAction)timesliderUpdate:(id)sender
+{
+    float f_updated;
+
+    switch( [[NSApp currentEvent] type] )
+    {
+        case NSLeftMouseUp:
+        case NSLeftMouseDown:
+            f_slider = [sender floatValue];
+            return;
+
+        case NSLeftMouseDragged:
+            f_updated = [sender floatValue];
+            break;
+
+        default:
+            return;
+    }
+
+    intf_thread_t * p_intf = [NSApp getIntf];
+
+    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
+                                                       FIND_ANYWHERE );
+
+    if( p_playlist == NULL )
+    {
+        return;
+    }
+
+    vlc_mutex_lock( &p_playlist->object_lock );
+
+    if( p_playlist->p_input != NULL )
+    {
+        off_t i_tell;
+        NSString * o_time;
+        char psz_time[ OFFSETTOTIME_MAX_SIZE ];
+
+#define p_area p_playlist->p_input->stream.p_selected_area
+        vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
+        i_tell = f_updated / 10000. * p_area->i_size;
+        input_OffsetToTime( p_playlist->p_input, psz_time, i_tell );
+        vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
+#undef p_area
+
+        o_time = [NSString stringWithUTF8String: psz_time];
+        [o_timefield setStringValue: o_time]; 
+    }
+
+    vlc_mutex_unlock( &p_playlist->object_lock );
+    vlc_object_release( p_playlist );
+}
+
 - (void)terminate
 {
     NSEvent * o_event;
@@ -961,12 +1069,6 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
         o_msg_lock = nil;
     }
 
-    if( o_prefs != nil )
-    {
-        [o_prefs release];
-        o_prefs = nil;
-    }
-
     [NSApp stop: nil];
 
     /* write cached user defaults to disk */
@@ -981,377 +1083,6 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
     [NSApp postEvent: o_event atStart: YES];
 }
 
-- (void)setupMenus:(input_thread_t *)p_input
-{
-    unsigned int i, i_nb_items;
-    NSMenuItem * o_item;
-    NSString * o_menu_title;
-    char psz_title[ 256 ];
-
-    es_descriptor_t * p_audio_es = NULL;
-    es_descriptor_t * p_spu_es = NULL;
-
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-    p_intf->p_sys->b_chapter_update |= p_intf->p_sys->b_title_update;
-    p_intf->p_sys->b_audio_update |= p_intf->p_sys->b_title_update |
-                                     p_intf->p_sys->b_program_update;
-    p_intf->p_sys->b_spu_update |= p_intf->p_sys->b_title_update |
-                                   p_intf->p_sys->b_program_update;
-
-    if( p_intf->p_sys->b_program_update )
-    {
-        NSMenu * o_program;
-        SEL pf_toggle_program;
-        pgrm_descriptor_t * p_pgrm;
-
-        if( p_input->stream.p_new_program )
-        {
-            p_pgrm = p_input->stream.p_new_program;
-        }
-        else
-        {
-            p_pgrm = p_input->stream.p_selected_program;
-        }
-
-        o_program = [o_mi_program submenu];
-        pf_toggle_program = @selector(toggleProgram:);
-
-        /* remove previous program items */
-        i_nb_items = [o_program numberOfItems];
-        for( i = 0; i < i_nb_items; i++ )
-        {
-            [o_program removeItemAtIndex: 0];
-        }
-
-        /* make (un)sensitive */
-        [o_mi_program setEnabled: 
-            p_input->stream.i_pgrm_number > 1];
-
-        /* add program items */
-        for( i = 0 ; i < p_input->stream.i_pgrm_number ; i++ )
-        {
-            snprintf( psz_title, sizeof(psz_title), "id %d",
-                p_input->stream.pp_programs[i]->i_number );
-            psz_title[sizeof(psz_title) - 1] = '\0';
-
-            o_menu_title = [NSApp localizedString: psz_title];
-
-            o_item = [o_program addItemWithTitle: o_menu_title
-                action: pf_toggle_program keyEquivalent: @""];
-            [o_item setTag: p_input->stream.pp_programs[i]->i_number];
-            [o_item setTarget: o_controls];
-
-            if( p_pgrm == p_input->stream.pp_programs[i] )
-            {
-                [o_item setState: NSOnState];
-            }
-        }
-
-        p_intf->p_sys->b_program_update = 0;
-    }
-
-    if( p_intf->p_sys->b_title_update )
-    {
-        NSMenu * o_title;
-        SEL pf_toggle_title;
-
-        o_title = [o_mi_title submenu];
-        pf_toggle_title = @selector(toggleTitle:);
-
-        /* remove previous title items */
-        i_nb_items = [o_title numberOfItems];
-        for( i = 0; i < i_nb_items; i++ )
-        {
-            [o_title removeItemAtIndex: 0];
-        }
-
-        /* make (un)sensitive */
-        [o_mi_title setEnabled: 
-            p_input->stream.i_area_nb > 1];
-
-        /* add title items */
-        for( i = 1 ; i < p_input->stream.i_area_nb ; i++ )
-        {
-            snprintf( psz_title, sizeof(psz_title), "Title %d (%d)", i,
-                p_input->stream.pp_areas[i]->i_part_nb );
-            psz_title[sizeof(psz_title) - 1] = '\0';
-
-            o_menu_title = [NSApp localizedString: psz_title];
-
-            o_item = [o_title addItemWithTitle: o_menu_title
-                action: pf_toggle_title keyEquivalent: @""];
-            [o_item setTag: i];
-            [o_item setTarget: o_controls];
-
-            if( ( p_input->stream.pp_areas[i] ==
-                p_input->stream.p_selected_area ) )
-            {
-                [o_item setState: NSOnState];
-            }
-        }
-
-        p_intf->p_sys->b_title_update = 0;
-    }
-
-    if( p_intf->p_sys->b_chapter_update )
-    {
-        NSMenu * o_chapter;
-        SEL pf_toggle_chapter;
-
-        o_chapter = [o_mi_chapter submenu];
-        pf_toggle_chapter = @selector(toggleChapter:);
-
-        /* remove previous chapter items */
-        i_nb_items = [o_chapter numberOfItems];
-        for( i = 0; i < i_nb_items; i++ )
-        {
-            [o_chapter removeItemAtIndex: 0];
-        }
-
-        /* make (un)sensitive */
-        [o_mi_chapter setEnabled: 
-            p_input->stream.p_selected_area->i_part_nb > 1];
-
-        /* add chapter items */
-        for( i = 0 ; i < p_input->stream.p_selected_area->i_part_nb ; i++ )
-        {
-            snprintf( psz_title, sizeof(psz_title), "Chapter %d", i + 1 );
-            psz_title[sizeof(psz_title) - 1] = '\0';
-
-            o_menu_title = [NSApp localizedString: psz_title];
-
-            o_item = [o_chapter addItemWithTitle: o_menu_title
-                action: pf_toggle_chapter keyEquivalent: @""];
-            [o_item setTag: i + 1];
-            [o_item setTarget: o_controls];
-
-            if( ( p_input->stream.p_selected_area->i_part == i + 1 ) )
-            {
-                [o_item setState: NSOnState];
-            }
-        }
-
-        p_intf->p_sys->i_part =
-                p_input->stream.p_selected_area->i_part;
-
-        p_intf->p_sys->b_chapter_update = 0;
-    }
-
-    for( i = 0 ; i < p_input->stream.i_selected_es_number ; i++ )
-    {
-        if( p_input->stream.pp_selected_es[i]->i_cat == AUDIO_ES )
-        {
-            p_audio_es = p_input->stream.pp_selected_es[i];
-        }
-        else if( p_input->stream.pp_selected_es[i]->i_cat == SPU_ES )
-        {
-            p_spu_es = p_input->stream.pp_selected_es[i];
-        }
-    }
-
-    if( p_intf->p_sys->b_audio_update )
-    {
-        [self setupLangMenu: p_input mi: o_mi_language es: p_audio_es
-            category: AUDIO_ES selector: @selector(toggleLanguage:)];
-
-        p_intf->p_sys->b_audio_update = 0;
-    }
-
-    if( p_intf->p_sys->b_spu_update )
-    {
-        [self setupLangMenu: p_input mi: o_mi_subtitle es: p_spu_es
-            category: SPU_ES selector: @selector(toggleLanguage:)];
-
-        p_intf->p_sys->b_spu_update = 0;
-    }
-
-    if ( p_intf->p_sys->b_aout_update )
-    {
-        aout_instance_t * p_aout = vlc_object_find( p_intf, VLC_OBJECT_AOUT,
-                                                    FIND_ANYWHERE );
-
-        if ( p_aout != NULL )
-        {
-            vlc_value_t val;
-            val.b_bool = 0;
-
-            var_Set( (vlc_object_t *)p_aout, "intf-change", val );
-
-            [self setupVarMenu: o_mi_channels target: (vlc_object_t *)p_aout
-                var: "audio-channels" selector: @selector(toggleVar:)];
-
-            [self setupVarMenu: o_mi_device target: (vlc_object_t *)p_aout
-                var: "audio-device" selector: @selector(toggleVar:)];
-
-            vlc_object_release( (vlc_object_t *)p_aout );
-        }
-
-        p_intf->p_sys->b_aout_update = 0;
-    }
-
-    if( p_intf->p_sys->b_vout_update )
-    {
-        vout_thread_t * p_vout = vlc_object_find( p_intf, VLC_OBJECT_VOUT,
-                                                          FIND_ANYWHERE );
-
-        if ( p_vout != NULL )
-        {
-            vlc_value_t val;
-            val.b_bool = 0;
-
-            var_Set( (vlc_object_t *)p_vout, "intf-change", val );
-
-            [self setupVarMenu: o_mi_screen target: (vlc_object_t *)p_vout
-                var: "video-device" selector: @selector(toggleVar:)];
-
-            vlc_object_release( (vlc_object_t *)p_vout );
-
-            [o_mi_close_window setEnabled: TRUE];
-        }
-
-        p_intf->p_sys->b_vout_update = 0;
-    }
-
-#undef p_input
-}
-
-- (void)setupLangMenu:(input_thread_t *)p_input
-                      mi:(NSMenuItem *)o_mi
-                      es:(es_descriptor_t *)p_es
-                      category:(int)i_cat
-                      selector:(SEL)pf_callback
-{
-    unsigned int i, i_nb_items;
-    NSMenu * o_menu = [o_mi submenu];
-
-    /* remove previous language items */
-    i_nb_items = [o_menu numberOfItems];
-    for( i = 0; i < i_nb_items; i++ )
-    {
-        [o_menu removeItemAtIndex: 0];
-    }
-
-    /* make sensitive : we can't change it after we build the menu, and
-     * before, we don't yet how many items we will have. So make it
-     * always sensitive. --Meuuh */
-    [o_mi setEnabled: TRUE];
-
-#if 0
-    /* We do not use this code, because you need to start stop .avi for
-     * it to work, so not very useful now  --hartman */
-    if ( o_mi == o_mi_subtitle ) {
-        NSLog(@"testing");
-        [o_mi setEnabled: TRUE ];
-        NSMenuItem * o_lmi;
-        NSString * o_title;
-        o_title = _NS("Load from file..");
-        o_lmi = [o_menu addItemWithTitle: o_title
-                 action: pf_callback keyEquivalent: @""];
-        [o_lmi setTag: 2000];
-        [o_lmi setTarget: o_controls];
-    }
-#endif
-
-#define ES p_input->stream.pp_es[i]
-    for( i = 0 ; i < p_input->stream.i_es_number ; i++ )
-    {
-        if( ( ES->i_cat == i_cat ) &&
-            ( !ES->p_pgrm ||
-              ES->p_pgrm ==
-                 p_input->stream.p_selected_program ) )
-        {
-            NSMenuItem * o_lmi;
-            NSString * o_title;
-
-            if( *ES->psz_desc )
-            {
-                o_title = [NSApp localizedString: ES->psz_desc];
-            }
-            else
-            {
-                char psz_title[ 256 ];
-
-                snprintf( psz_title, sizeof(psz_title), _("Language 0x%x"),
-                          ES->i_id );
-                psz_title[sizeof(psz_title) - 1] = '\0';
-
-                o_title = [NSApp localizedString: psz_title];
-            }
-
-            o_lmi = [o_menu addItemWithTitle: o_title
-                action: pf_callback keyEquivalent: @""];
-            [o_lmi setRepresentedObject: 
-                [NSValue valueWithPointer: ES]];
-            [o_lmi setTarget: o_controls];
-            [o_lmi setTag: i_cat];
-
-            if( /*p_es == ES*/ ES->p_decoder_fifo != NULL )
-            {
-                [o_lmi setState: NSOnState];
-            }
-        }
-    }
-#undef ES
-}
-
-- (void)setupVarMenu:(NSMenuItem *)o_mi
-                     target:(vlc_object_t *)p_object
-                     var:(const char *)psz_variable
-                     selector:(SEL)pf_callback
-{
-    int i, i_nb_items;
-    NSMenu * o_menu = [o_mi submenu];
-    vlc_value_t val;
-    char * psz_value;
-
-    /* remove previous items */
-    i_nb_items = [o_menu numberOfItems];
-    for( i = 0; i < i_nb_items; i++ )
-    {
-        [o_menu removeItemAtIndex: 0];
-    }
-
-    if ( var_Get( p_object, psz_variable, &val ) < 0 )
-    {
-        return;
-    }
-    psz_value = val.psz_string;
-
-    if ( var_Change( p_object, psz_variable,
-                     VLC_VAR_GETLIST, &val ) < 0 )
-    {
-        free( psz_value );
-        return;
-    }
-
-    /* make (un)sensitive */
-    [o_mi setEnabled: ( val.p_list->i_count > 0 )];
-
-    for ( i = 0; i < val.p_list->i_count; i++ )
-    {
-        NSMenuItem * o_lmi;
-        NSString * o_title;
-
-        o_title = [NSApp localizedString: val.p_list->p_values[i].psz_string];
-        o_lmi = [o_menu addItemWithTitle: o_title
-                 action: pf_callback keyEquivalent: @""];
-        /* FIXME: this isn't 64-bit clean ! */
-        [o_lmi setTag: (int)psz_variable];
-        [o_lmi setRepresentedObject:
-            [NSValue valueWithPointer: p_object]];
-        [o_lmi setTarget: o_controls];
-
-        if ( !strcmp( psz_value, val.p_list->p_values[i].psz_string ) )
-            [o_lmi setState: NSOnState];
-    }
-
-    var_Change( p_object, psz_variable, VLC_VAR_FREELIST,
-                &val );
-
-    free( psz_value );
-}
-
 - (IBAction)clearRecentItems:(id)sender
 {
     [[NSDocumentController sharedDocumentController]
@@ -1365,64 +1096,7 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
 - (IBAction)viewPreferences:(id)sender
 {
-    if( o_prefs == nil )
-    {
-        o_prefs = [[VLCPrefs alloc] init];
-    }
-
-    [o_prefs createPrefPanel: @"main"];
-}
-
-- (IBAction)timesliderUpdate:(id)sender
-{
-    float f_updated;
-
-    switch( [[NSApp currentEvent] type] )
-    {
-        case NSLeftMouseUp:
-        case NSLeftMouseDown:
-            f_slider = [sender floatValue];
-            return;
-
-        case NSLeftMouseDragged:
-            f_updated = [sender floatValue];
-            break;
-
-        default:
-            return;
-    }
-
-    intf_thread_t * p_intf = [NSApp getIntf];
-
-    playlist_t * p_playlist = vlc_object_find( p_intf, VLC_OBJECT_PLAYLIST,
-                                                       FIND_ANYWHERE );
-
-    if( p_playlist == NULL )
-    {
-        return;
-    }
-
-    vlc_mutex_lock( &p_playlist->object_lock );
-
-    if( p_playlist->p_input != NULL )
-    {
-        off_t i_tell;
-        NSString * o_time;
-        char psz_time[ OFFSETTOTIME_MAX_SIZE ];
-
-#define p_area p_playlist->p_input->stream.p_selected_area
-        vlc_mutex_lock( &p_playlist->p_input->stream.stream_lock );
-        i_tell = f_updated / 10000. * p_area->i_size;
-        input_OffsetToTime( p_playlist->p_input, psz_time, i_tell );
-        vlc_mutex_unlock( &p_playlist->p_input->stream.stream_lock );
-#undef p_area
-
-        o_time = [NSString stringWithCString: psz_time];
-        [o_timefield setStringValue: o_time]; 
-    }
-
-    vlc_mutex_unlock( &p_playlist->object_lock );
-    vlc_object_release( p_playlist );
+    [o_prefs showPrefs];
 }
 
 - (IBAction)closeError:(id)sender
@@ -1440,6 +1114,14 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
                                    withApplication: @"TextEdit"];
 }
 
+- (IBAction)openDocumentation:(id)sender
+{
+    NSURL * o_url = [NSURL URLWithString: 
+        @"http://www.videolan.org/doc/"];
+
+    [[NSWorkspace sharedWorkspace] openURL: o_url];
+}
+
 - (IBAction)reportABug:(id)sender
 {
     NSURL * o_url = [NSURL URLWithString: 
@@ -1510,11 +1192,17 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
 
 - (BOOL)validateMenuItem:(NSMenuItem *)o_mi
 {
+    NSString *o_title = [o_mi title];
     BOOL bEnabled = TRUE;
 
-    /* Recent Items Menu */
+    if( [o_title isEqualToString: _NS("License")] )
+    {
+        /* we need to do this only once */
+        [self setupMenus];
+    }
 
-    if( [[o_mi title] isEqualToString: _NS("Clear Menu")] )
+    /* Recent Items Menu */
+    if( [o_title isEqualToString: _NS("Clear Menu")] )
     {
         NSMenu * o_menu = [o_mi_open_recent submenu];
         int i_nb_items = [o_menu numberOfItems];
@@ -1567,7 +1255,6 @@ int ExecuteOnMainThread( id target, SEL sel, void * p_arg )
             bEnabled = FALSE;
         }
     }
-
     return( bEnabled );
 }
 
