@@ -2,7 +2,7 @@
  * open.m: MacOS X plugin for vlc
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: open.m,v 1.27 2003/04/01 22:29:41 massiot Exp $
+ * $Id: open.m,v 1.35 2003/05/25 17:27:13 massiot Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net> 
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -41,8 +41,6 @@
 #include "playlist.h"
 #include "open.h"
 
-#include "netutils.h"
-
 /*****************************************************************************
  * GetEjectableMediaOfClass 
  *****************************************************************************/
@@ -132,17 +130,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
 - (void)awakeFromNib
 {
     intf_thread_t * p_intf = [NSApp getIntf];
-    char * psz_sout = config_GetPsz( p_intf, "sout" );
-
-    if ( psz_sout != NULL && *psz_sout )
-    {
-        [o_sout_cbox setState: YES];
-    }
-    else
-    {
-        [o_sout_cbox setState: NO];
-    }
-    free(psz_sout);
 
     [o_panel setTitle: _NS("Open Source")];
     [o_mrl_lbl setTitle: _NS("Media Resource Locator (MRL)")];
@@ -162,11 +149,12 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
     [o_disc_title_lbl setStringValue: _NS("Title")];
     [o_disc_chapter_lbl setStringValue: _NS("Chapter")];
     [o_disc_videots_btn_browse setTitle: _NS("Browse...")];
-    [o_disc_dvd_menus setTitle: _NS("Use DVD menus (EXPERIMENTAL)")];
+    [o_disc_dvd_menus setTitle: _NS("Use DVD menus")];
 
     [[o_disc_type cellAtRow:0 column:0] setTitle: _NS("VIDEO_TS folder")];
     [[o_disc_type cellAtRow:1 column:0] setTitle: _NS("DVD")];
     [[o_disc_type cellAtRow:2 column:0] setTitle: _NS("VCD")];
+    [[o_disc_type cellAtRow:3 column:0] setTitle: _NS("Audio CD")];
 
     [o_net_udp_port_lbl setStringValue: _NS("Port")];
     [o_net_udpm_addr_lbl setStringValue: _NS("Address")];
@@ -180,27 +168,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
     [o_net_udp_port setIntValue: config_GetInt( p_intf, "server-port" )];
     [o_net_udp_port_stp setIntValue: config_GetInt( p_intf, "server-port" )];
 
-    [o_sout_cbox setTitle: _NS("Stream output:")];
-    [o_sout_btn_ok setTitle: _NS("OK")];
-    [o_sout_settings setTitle: _NS("Settings...")];
-    [o_sout_mrl_lbl setTitle: _NS("Stream output MRL")];
-    
-    [o_sout_access_lbl setTitle: _NS("Output Method")];
-    [[o_sout_access cellAtRow:0 column:0] setTitle: _NS("File")];
-    [[o_sout_access cellAtRow:1 column:0] setTitle: _NS("HTTP")];
-    [[o_sout_access cellAtRow:2 column:0] setTitle: _NS("UDP")];
-    [[o_sout_access cellAtRow:3 column:0] setTitle: _NS("RTP")];
-
-    [o_sout_file_btn_browse setTitle: _NS("Browse...")];
-    [o_sout_udp_addr_lbl setStringValue: _NS("Address")];
-    [o_sout_udp_port_lbl setStringValue: _NS("Port")];
-
-    [o_sout_mux_lbl setTitle: _NS("Encapsulation Method")];
-    [[o_sout_mux cellAtRow:0 column:0] setTitle: _NS("MPEG TS")];
-    [[o_sout_mux cellAtRow:0 column:1] setTitle: _NS("MPEG PS")];
-    [[o_sout_mux cellAtRow:0 column:2] setTitle: _NS("AVI")];
-    [[o_sout_mux cellAtRow:0 column:3] setTitle: _NS("Ogg")];
-    
     [o_file_sub_ckbox setTitle: _NS("Load subtitles file:")];
     [o_file_sub_btn_settings setTitle: _NS("Settings...")];
     [o_file_sub_btn_browse setTitle: _NS("Browse...")];
@@ -249,19 +216,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
         selector: @selector(openNetInfoChanged:)
         name: NSControlTextDidChangeNotification
         object: o_net_http_url];
-
-    [[NSNotificationCenter defaultCenter] addObserver: self
-        selector: @selector(soutInfoChanged:)
-        name: NSControlTextDidChangeNotification
-        object: o_sout_file_path];
-    [[NSNotificationCenter defaultCenter] addObserver: self
-        selector: @selector(soutInfoChanged:)
-        name: NSControlTextDidChangeNotification
-        object: o_sout_udp_addr];
-    [[NSNotificationCenter defaultCenter] addObserver: self
-        selector: @selector(soutInfoChanged:)
-        name: NSControlTextDidChangeNotification
-        object: o_sout_udp_port];
 }
 
 - (void)openTarget:(int)i_type
@@ -269,7 +223,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
     int i_result;
 
     [o_tabview selectTabViewItemAtIndex: i_type];
-    [o_ckbox_enqueue setState: NSOnState];
     [o_file_sub_ckbox setState: NSOffState];
     
     i_result = [NSApp runModalForWindow: o_panel];
@@ -277,24 +230,15 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
 
     if( i_result )
     {
-        NSString *o_sout = [o_sout_mrl stringValue];
         intf_thread_t * p_intf = [NSApp getIntf];
-        
-        if ( [o_sout_cbox state] )
-        {
-            config_PutPsz( p_intf, "sout", [o_sout lossyCString] );
-        }
 
         NSString *o_source = [o_mrl stringValue];
         BOOL b_enq = [o_ckbox_enqueue state] == NSOnState ? YES : NO;
         NSString *subPath = [o_file_sub_path stringValue];
         
-        [o_playlist appendArray: 
-            [NSArray arrayWithObject: o_source] atPos: -1 enqueue:b_enq];
-        
         if (([o_file_sub_ckbox state] == NSOnState) && !([subPath isEqualTo: @""]))
         {
-            config_PutPsz( p_intf, "sub-file", strdup( [subPath cString] ) );
+            config_PutPsz( p_intf, "sub-file", strdup( [subPath UTF8String] ) );
             if ( [o_file_sub_override state] )
             {
                 config_PutInt( p_intf, "sub-delay", (int)( [o_file_sub_delay intValue] * 10 ) );
@@ -307,8 +251,10 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
             config_PutInt( p_intf, "sub-delay", 0 );
             config_PutFloat( p_intf, "sub-fps", 0.0 );
         }
+        
+        [o_playlist appendArray: 
+            [NSArray arrayWithObject: o_source] atPos: -1 enqueue:b_enq];
     }
-    [self soutModeChanged: nil];
 }
 
 - (void)tabView:(NSTabView *)o_tv didSelectTabViewItem:(NSTabViewItem *)o_tvi
@@ -440,6 +386,13 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
             b_menus = 0; b_title_chapter = 1;
             [o_disc_dvd_menus setState: FALSE];
         }
+        else if ( [o_type isEqualToString: _NS("Audio CD")])
+        {
+            psz_class = kIOCDMediaClass;
+            o_disc = o_type;
+            b_menus = 0; b_title_chapter = 0;
+            [o_disc_dvd_menus setState: FALSE];
+        }
         else
         {
             psz_class = kIODVDMediaClass;
@@ -524,6 +477,14 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
         o_mrl_string = [NSString stringWithFormat: @"vcd://%@@%i,%i",
                         o_device, i_title, i_chapter]; 
     }
+    else if ( [o_type isEqualToString: _NS("Audio CD")] )
+    {
+        if ( [o_device isEqualToString:
+                [NSString stringWithFormat: _NS("No %@s found"), o_type]] )
+            o_device = @"";
+        o_mrl_string = [NSString stringWithFormat: @"cdda://%@",
+                        o_device]; 
+    }
     else if ( [o_type isEqualToString: _NS("DVD")] )
     {
         if ( [o_device isEqualToString:
@@ -660,140 +621,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
     [o_mrl setStringValue: o_mrl_string];
 }
 
-- (IBAction)soutChanged:(id)sender;
-{
-    if ([o_sout_cbox state] == NSOnState)
-    {
-        [o_sout_settings setEnabled:YES];
-    }
-    else
-    {
-        [o_sout_settings setEnabled:NO];
-    }
-}
-
-- (IBAction)soutSettings:(id)sender
-{
-    [self soutModeChanged: nil];
-    [NSApp beginSheet: o_sout_sheet
-        modalForWindow: [sender window]
-        modalDelegate: self
-        didEndSelector: NULL
-        contextInfo: nil];
-}
-
-- (IBAction)soutFileBrowse:(id)sender
-{
-    NSSavePanel *o_save_panel = [NSSavePanel savePanel];
-    NSString *o_mux_string;
-    if ( [[[o_sout_mux selectedCell] title] isEqualToString: _NS("MPEG PS")] )
-        o_mux_string = @"vob";
-    else if ( [[[o_sout_mux selectedCell] title] isEqualToString: _NS("AVI")] )
-        o_mux_string = @"avi";
-    else if ( [[[o_sout_mux selectedCell] title] isEqualToString: _NS("Ogg")] )
-        o_mux_string = @"ogm";
-    else
-        o_mux_string = @"ts";
-
-    NSString * o_name = [NSString stringWithFormat: @"vlc-output.%@",
-                         o_mux_string];
-
-    [o_save_panel setTitle: _NS("Save File")];
-    [o_save_panel setPrompt: _NS("Save")];
-
-    if( [o_save_panel runModalForDirectory: nil
-            file: o_name] == NSOKButton )
-    {
-        NSString *o_filename = [o_save_panel filename];
-        [o_sout_file_path setStringValue: o_filename];
-        [self soutInfoChanged: nil];
-    }
-}
-
-- (void)soutModeChanged:(NSNotification *)o_notification
-{
-    NSString *o_mode;
-    BOOL b_file = FALSE;
-    BOOL b_net = FALSE;
-
-    o_mode = [[o_sout_access selectedCell] title];
-
-    if( [o_mode isEqualToString: _NS("File")] ) b_file = TRUE;   
-    else if( [o_mode isEqualToString: _NS("UDP")] ) b_net = TRUE;
-    else if( [o_mode isEqualToString: _NS("RTP")] ) b_net = TRUE;
-
-    [o_sout_file_path setEnabled: b_file];
-    [o_sout_file_btn_browse setEnabled: b_file];
-    [o_sout_udp_addr setEnabled: !b_file];
-    [o_sout_udp_port setEnabled: !b_file];
-    [o_sout_udp_port_stp setEnabled: !b_file ];
-    
-    [[o_sout_mux cellAtRow:0 column: 1] setEnabled: !b_net];
-    [[o_sout_mux cellAtRow:0 column: 2] setEnabled: !b_net];
-    [[o_sout_mux cellAtRow:0 column: 3] setEnabled: !b_net];
-    if ( b_net )
-    {
-        [o_sout_mux selectCellAtRow:0 column: 0];
-    }
-
-    [self soutInfoChanged: nil];
-}
-
-- (void)soutInfoChanged:(NSNotification *)o_notification
-{
-    NSString *o_mode;
-    NSString *o_mux;
-    NSString *o_mrl_string;
-    NSString *o_mux_string;
-
-    o_mode = [[o_sout_access selectedCell] title];
-    o_mux = [[o_sout_mux selectedCell] title];
-
-    if ( [o_mux isEqualToString: _NS("AVI")] ) o_mux_string = @"avi";
-    else if ( [o_mux isEqualToString: _NS("Ogg")] ) o_mux_string = @"ogg";
-    else if ( [o_mux isEqualToString: _NS("MPEG PS")] ) o_mux_string = @"ps";
-    else o_mux_string = @"ts";
-
-    if ( [o_mode isEqualToString: _NS("File")] )
-    {
-        o_mrl_string = [NSString stringWithFormat: @"file/%@://%@",
-                        o_mux_string, [o_sout_file_path stringValue]];
-    }
-    else if ( [o_mode isEqualToString: _NS("HTTP")] )
-    {
-        o_mrl_string = [NSString stringWithFormat: @"http/%@://%@:%i",
-                        o_mux_string, [o_sout_udp_addr stringValue],
-                        [o_sout_udp_port intValue]];
-    }
-    else if ( [o_mode isEqualToString: _NS("UDP")] )
-    {
-        o_mrl_string = [NSString stringWithFormat: @"udp/%@://%@:%i",
-                        o_mux_string, [o_sout_udp_addr stringValue],
-                        [o_sout_udp_port intValue]];
-    }
-    else
-    {
-        o_mrl_string = [NSString stringWithFormat: @"rtp/%@://%@:%i",
-                        o_mux_string, [o_sout_udp_addr stringValue],
-                        [o_sout_udp_port intValue]];
-    }
-
-
-    [o_sout_mrl setStringValue: o_mrl_string];
-}
-
-- (IBAction)soutStepperChanged:(id)sender
-{
-    [o_sout_udp_port setIntValue: [o_sout_udp_port_stp intValue]];
-    [self soutInfoChanged: nil];
-}
-
-- (IBAction)soutCloseSheet:(id)sender
-{
-    [o_sout_sheet orderOut:sender];
-    [NSApp endSheet: o_sout_sheet];
-}
-
 - (IBAction)openFile:(id)sender
 {
     NSOpenPanel *o_open_panel = [NSOpenPanel openPanel];
@@ -808,7 +635,16 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
     {
         intf_thread_t * p_intf = [NSApp getIntf];
         config_PutPsz( p_intf, "sout", NULL );
-        [o_playlist appendArray: [o_open_panel filenames] atPos: -1 enqueue:NO];
+        
+        NSArray *o_values = [[o_open_panel filenames]
+                sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
+        
+        config_PutPsz( p_intf, "sub-file", "" );
+        config_PutInt( p_intf, "sub-delay", 0 );
+        config_PutFloat( p_intf, "sub-fps", 0.0 );
+        config_PutPsz( p_intf, "sout", "" );
+        
+        [o_playlist appendArray: o_values atPos: -1 enqueue:NO];
     }
 }
 
