@@ -2,7 +2,7 @@
  * playlist.m: MacOS X interface plugin
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: playlist.m,v 1.17 2003/03/17 23:13:06 hartman Exp $
+ * $Id: playlist.m,v 1.25 2003/06/02 12:42:15 hartman Exp $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Derk-Jan Hartman <thedj@users.sourceforge.net>
@@ -111,7 +111,7 @@
     }
 }
 
-- (void) highlightSelectionInClipRect:(NSRect)o_rect {
+- (void)highlightSelectionInClipRect:(NSRect)o_rect {
     NSRect o_new_rect;
     float f_height = [self rowHeight] + [self intercellSpacing].height;
     float f_origin_y = NSMaxY( o_rect );
@@ -131,6 +131,12 @@
     {
         o_striped_row_color = [[[NSColor alternateSelectedControlColor]
                                 highlightWithLevel: 0.90] retain];
+        
+        if ( o_striped_row_color == nil )
+        {
+            /* OSX 10.1 and before ain't that smart ;) */
+            o_striped_row_color = [[NSColor whiteColor] retain];
+        }
     }
     [o_striped_row_color set];
     
@@ -250,6 +256,7 @@
      * when the playlist changes. we do this on purpose, because else there is a 
      * delay of .5 sec or so when we delete an item */
     [self playlistUpdated];
+    [self updateRowSelection];
 }
 
 - (IBAction)selectAll:(id)sender
@@ -442,19 +449,27 @@
         NSArray * o_values;
         NSPasteboard * o_pasteboard;
         
+        intf_thread_t * p_intf = [NSApp getIntf];
         o_pasteboard = [o_info draggingPasteboard];
         
         if( [[o_pasteboard types] containsObject: NSFilenamesPboardType] )
         {
-            o_values = [o_pasteboard propertyListForType: NSFilenamesPboardType];
-        
+            o_values = [[o_pasteboard propertyListForType: NSFilenamesPboardType]
+                        sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
+
+            config_PutPsz( p_intf, "sub-file", "" );
+            config_PutInt( p_intf, "sub-delay", 0 );
+            config_PutFloat( p_intf, "sub-fps", 0.0 );
+            config_PutPsz( p_intf, "sout", "" );
+
             [self appendArray: o_values atPos: i_proposed_row enqueue:YES];
-        
+
             return( YES );
         }
         
         return( NO );
     }
+    [self updateRowSelection];
 }
 
 @end
