@@ -2,7 +2,7 @@
  * os_theme.h: Wrapper for the OSTheme class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: os_theme.h,v 1.1 2003/03/18 02:21:47 ipkiss Exp $
+ * $Id: os_theme.h,v 1.5 2003/04/28 14:12:32 asmax Exp $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -25,6 +25,12 @@
 
 
 #if defined( WIN32 )
-    #include "win32_theme.h"
+    #include "win32/win32_theme.h"
     #define OSTheme Win32Theme
+#elif defined GTK2_SKINS
+    #include "gtk2/gtk2_theme.h"
+    #define OSTheme GTK2Theme
+#elif defined X11_SKINS
+    #include "x11/x11_theme.h"
+    #define OSTheme X11Theme
 #endif
