@@ -2,7 +2,7 @@
  * wxwindows.h: private wxWindows interface description
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: wxwindows.h,v 1.16 2003/04/06 16:30:43 gbazin Exp $
+ * $Id: wxwindows.h,v 1.36 2003/06/12 21:28:39 gbazin Exp $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -26,14 +26,25 @@
 #include <wx/notebook.h>
 #include <wx/spinctrl.h>
 #include <wx/dnd.h>
+#include <wx/treectrl.h>
+#include <wx/gauge.h>
 
 class Playlist;
 class Messages;
+class FileInfo;
 
 #define SLIDER_MAX_POS 10000
 
+/* wxU is used to convert ansi strings to unicode strings (wchar_t) */
+#if wxUSE_UNICODE
+#   define wxU(ansi) wxString(ansi, *wxConvCurrent)
+#else
+#   define wxU(ansi) ansi
+#endif
+
+#if !defined(MODULE_NAME_IS_skins)
 /*****************************************************************************
- * intf_sys_t: description and status of Gtk+ interface
+ * intf_sys_t: description and status of wxwindows interface
  *****************************************************************************/
 struct intf_sys_t
 {
@@ -44,24 +55,10 @@ struct intf_sys_t
     /* secondary windows */
     Playlist            *p_playlist_window;
     Messages            *p_messages_window;
+    FileInfo            *p_fileinfo_window;
 
     /* special actions */
     vlc_bool_t          b_playing;
-    vlc_bool_t          b_popup_changed;                   /* display menu ? */
-    vlc_bool_t          b_window_changed;        /* window display toggled ? */
-    vlc_bool_t          b_playlist_changed;    /* playlist display toggled ? */
-    vlc_bool_t          b_slider_free;                      /* slider status */
-
-    /* menus handlers */
-    vlc_bool_t          b_program_update;   /* do we need to update programs
-                                                                        menu */
-    vlc_bool_t          b_title_update;  /* do we need to update title menus */
-    vlc_bool_t          b_chapter_update;            /* do we need to update
-                                                               chapter menus */
-    vlc_bool_t          b_audio_update;  /* do we need to update audio menus */
-    vlc_bool_t          b_spu_update;      /* do we need to update spu menus */
-
-    /* windows and widgets */
 
     /* The input thread */
     input_thread_t *    p_input;
@@ -69,6 +66,7 @@ struct intf_sys_t
     /* The slider */
     int                 i_slider_pos;                     /* slider position */
     int                 i_slider_oldpos;                /* previous position */
+    vlc_bool_t          b_slider_free;                      /* slider status */
 
     /* The messages window */
     msg_subscription_t* p_sub;                  /* message bank subscription */
@@ -76,9 +74,12 @@ struct intf_sys_t
     /* Playlist management */
     int                 i_playing;                 /* playlist selected item */
 
-    /* The window labels for DVD mode */
-    unsigned int        i_part;                           /* current chapter */
+    /* Popup menu */
+    wxMenu              *p_popup_menu;
+    vlc_bool_t          b_popup_change;
+
 };
+#endif /* !defined(MODULE_NAME_IS_skins) */
 
 /*****************************************************************************
  * Prototypes
@@ -103,9 +104,11 @@ private:
     intf_thread_t *p_intf;
     Interface *p_main_interface;
     int i_old_playing_status;
+    int i_old_rate;
 };
 
 /* Main Interface */
+class OpenDialog;
 class Interface: public wxFrame
 {
 public:
@@ -121,9 +124,11 @@ public:
     wxWindow    *slider_frame;
     wxStaticBox *slider_box;
 
-    wxMenu      *p_popup_menu;
+    wxGauge     *volctrl;
 
-    wxArrayString mrl_history;
+    /* So we don't recreate the open dialog box each time
+     * (and keep the last settings) */
+    OpenDialog  *p_open_dialog;
 
 private:
     void CreateOurMenuBar();
@@ -150,6 +155,15 @@ private:
     void OnSliderUpdate( wxScrollEvent& event );
     void OnPrevStream( wxCommandEvent& event );
     void OnNextStream( wxCommandEvent& event );
+    void OnSlowStream( wxCommandEvent& event );
+    void OnFastStream( wxCommandEvent& event );
+
+    void OnMenuOpen( wxMenuEvent& event );
+
+#if defined( __WXMSW__ ) || defined( __WXMAC__ )
+    void OnContextMenu2(wxContextMenuEvent& event);
+#endif
+    void OnContextMenu(wxMouseEvent& event);
 
     DECLARE_EVENT_TABLE();
 
@@ -159,20 +173,31 @@ private:
     wxFrame *p_prefs_dialog;
 
     int i_old_playing_status;
+
+    /* For auto-generated menus */
+    wxMenu *p_audio_menu;
+    vlc_bool_t b_audio_menu;
+    wxMenu *p_video_menu;
+    vlc_bool_t b_video_menu;
+    wxMenu *p_navig_menu;
+    vlc_bool_t b_navig_menu;
 };
 
 /* Open Dialog */
+class SoutDialog;
+class SubsFileDialog;
 class OpenDialog: public wxDialog
 {
 public:
     /* Constructor */
-    OpenDialog( intf_thread_t *p_intf, Interface *p_main_interface,
+    OpenDialog( intf_thread_t *p_intf, wxWindow *p_parent,
                 int i_access_method );
     virtual ~OpenDialog();
-    void Rebuild();
-    void Manage();
 
-    wxString mrl;
+    int ShowModal();
+    int ShowModal( int i_access_method );
+
+    wxArrayString mrl;
 
 private:
     wxPanel *FilePanel( wxWindow* parent );
@@ -181,6 +206,7 @@ private:
     wxPanel *SatPanel( wxWindow* parent );
 
     void UpdateMRL( int i_access_method );
+    wxArrayString SeparateEntries( wxString );
 
     /* Event handlers (these functions should _not_ be virtual) */
     void OnOk( wxCommandEvent& event );
@@ -201,6 +227,10 @@ private:
     void OnNetPanelChange( wxCommandEvent& event );
     void OnNetTypeChange( wxCommandEvent& event );
 
+    /* Event handlers for the stream output */
+    void OnSubsFileEnable( wxCommandEvent& event );
+    void OnSubsFileSettings( wxCommandEvent& WXUNUSED(event) );
+
     /* Event handlers for the stream output */
     void OnSoutEnable( wxCommandEvent& event );
     void OnSoutSettings( wxCommandEvent& WXUNUSED(event) );
@@ -213,13 +243,15 @@ private:
     DECLARE_EVENT_TABLE();
 
     intf_thread_t *p_intf;
-    Interface *p_main_interface;
+    wxWindow *p_parent;
     int i_current_access_method;
 
     wxComboBox *mrl_combo;
+    wxNotebook *notebook;
 
     /* Controls for the file panel */
     wxComboBox *file_combo;
+    wxFileDialog *file_dialog;
 
     /* Controls for the disc panel */
     wxRadioBox *disc_type;
@@ -235,14 +267,21 @@ private:
     wxSpinCtrl *net_ports[4];
     wxTextCtrl *net_addrs[4];
 
+    /* Controls for the subtitles file */
+    wxButton *subsfile_button;
+    wxCheckBox *subsfile_checkbox;
+    SubsFileDialog *subsfile_dialog;
+
     /* Controls for the stream output */
     wxButton *sout_button;
     wxCheckBox *sout_checkbox;
+    SoutDialog *sout_dialog;
 
     /* Controls for the demux dump */
     wxTextCtrl *demuxdump_textctrl;
     wxButton *demuxdump_button;
     wxCheckBox *demuxdump_checkbox;
+    wxFileDialog *demuxdump_dialog;
 };
 
 enum
@@ -258,7 +297,7 @@ class SoutDialog: public wxDialog
 {
 public:
     /* Constructor */
-    SoutDialog( intf_thread_t *p_intf, Interface *p_main_interface );
+    SoutDialog( intf_thread_t *p_intf, wxWindow *p_parent );
     virtual ~SoutDialog();
 
     wxString mrl;
@@ -267,6 +306,7 @@ private:
     void UpdateMRL();
     wxPanel *AccessPanel( wxWindow* parent );
     wxPanel *EncapsulationPanel( wxWindow* parent );
+    wxPanel *TranscodingPanel( wxWindow* parent );
     void    ParseMRL();
 
     /* Event handlers (these functions should _not_ be virtual) */
@@ -285,29 +325,62 @@ private:
     /* Event handlers for the encapsulation panel */
     void OnEncapsulationChange( wxCommandEvent& event );
 
+    /* Event handlers for the transcoding panel */
+    void OnTranscodingEnable( wxCommandEvent& event );
+    void OnTranscodingChange( wxCommandEvent& event );
+
     DECLARE_EVENT_TABLE();
 
     intf_thread_t *p_intf;
-    Interface *p_main_interface;
+    wxWindow *p_parent;
 
     wxComboBox *mrl_combo;
-    wxPanel *access_panel;
-    wxPanel *encapsulation_panel;
 
     /* Controls for the access outputs */
-    wxPanel *access_subpanels[4];
-    wxRadioButton *access_radios[4];
+    wxPanel *access_subpanels[5];
+    wxCheckBox *access_checkboxes[5];
 
     int i_access_type;
 
     wxComboBox *file_combo;
-    wxSpinCtrl *net_port;
-    wxTextCtrl *net_addr;
+    wxSpinCtrl *net_ports[5];
+    wxTextCtrl *net_addrs[5];
 
     /* Controls for the encapsulation */
-    wxRadioButton *encapsulation_radios[4];
+    wxRadioButton *encapsulation_radios[5];
     int i_encapsulation_type;
 
+    /* Controls for transcoding */
+    wxCheckBox *video_transc_checkbox;
+    wxComboBox *video_codec_combo;
+    wxComboBox *audio_codec_combo;
+    wxCheckBox *audio_transc_checkbox;
+    wxComboBox *video_bitrate_combo;
+    wxComboBox *audio_bitrate_combo;
+};
+
+/* Subtitles File Dialog */
+class SubsFileDialog: public wxDialog
+{
+public:
+    /* Constructor */
+    SubsFileDialog( intf_thread_t *p_intf, wxWindow *p_parent );
+    virtual ~SubsFileDialog();
+
+    wxComboBox *file_combo;
+    wxSpinCtrl *delay_spinctrl;
+    wxSpinCtrl *fps_spinctrl;
+
+private:
+    /* Event handlers (these functions should _not_ be virtual) */
+    void OnOk( wxCommandEvent& event );
+    void OnCancel( wxCommandEvent& event );
+    void OnFileBrowse( wxCommandEvent& event );
+
+    DECLARE_EVENT_TABLE();
+
+    intf_thread_t *p_intf;
+    wxWindow *p_parent;
 };
 
 /* Preferences Dialog */
@@ -316,7 +389,7 @@ class PrefsDialog: public wxFrame
 {
 public:
     /* Constructor */
-    PrefsDialog( intf_thread_t *p_intf, Interface *p_main_interface );
+    PrefsDialog( intf_thread_t *p_intf, wxWindow *p_parent );
     virtual ~PrefsDialog();
 
 private:
@@ -326,11 +399,11 @@ private:
     void OnOk( wxCommandEvent& event );
     void OnCancel( wxCommandEvent& event );
     void OnSave( wxCommandEvent& event );
+    void OnResetAll( wxCommandEvent& event );
 
     DECLARE_EVENT_TABLE();
 
     intf_thread_t *p_intf;
-    Interface *p_main_interface;
 
     PrefsTreeCtrl *prefs_tree;
 };
@@ -340,7 +413,7 @@ class Messages: public wxFrame
 {
 public:
     /* Constructor */
-    Messages( intf_thread_t *p_intf, Interface *_p_main_interface );
+    Messages( intf_thread_t *p_intf, wxWindow *p_parent );
     virtual ~Messages();
     void UpdateLog();
 
@@ -352,7 +425,6 @@ private:
     DECLARE_EVENT_TABLE();
 
     intf_thread_t *p_intf;
-    Interface *p_main_interface;
     wxTextCtrl *textctrl;
     wxTextAttr *info_attr;
     wxTextAttr *err_attr;
@@ -369,28 +441,34 @@ public:
     /* Constructor */
     Playlist( intf_thread_t *p_intf, Interface *p_main_interface );
     virtual ~Playlist();
-    void Rebuild();
-    void Manage();
+
+    void UpdatePlaylist();
+    void ShowPlaylist( bool show );
+
+    bool b_need_update;
+    vlc_mutex_t lock;
 
 private:
     void DeleteItem( int item );
 
     /* Event handlers (these functions should _not_ be virtual) */
-    void OnAddUrl( wxCommandEvent& event );
-    void OnAddDirectory( wxCommandEvent& event );
+    void OnAddMRL( wxCommandEvent& event );
     void OnClose( wxCommandEvent& event );
+    void OnOpen( wxCommandEvent& event );
+    void OnSave( wxCommandEvent& event );
     void OnInvertSelection( wxCommandEvent& event );
     void OnDeleteSelection( wxCommandEvent& event );
     void OnSelectAll( wxCommandEvent& event );
     void OnActivateItem( wxListEvent& event );
     void OnKeyDown( wxListEvent& event );
+    void Rebuild();
 
     DECLARE_EVENT_TABLE();
 
     intf_thread_t *p_intf;
     Interface *p_main_interface;
     wxListView *listview;
-    wxButton *ok_button;
+    int i_update_counter;
 };
 
 /* File Info */
@@ -398,13 +476,20 @@ class FileInfo: public wxFrame
 {
 public:
     /* Constructor */
-    FileInfo( intf_thread_t *p_intf, Interface *p_main_interface );
+    FileInfo( intf_thread_t *p_intf, wxWindow *p_parent );
     virtual ~FileInfo();
+    void UpdateFileInfo();
 
 private:
     void OnClose( wxCommandEvent& event );
+
     DECLARE_EVENT_TABLE();
-    
+   
+    intf_thread_t *p_intf;
+    wxTreeCtrl *fileinfo_tree;
+    wxTreeItemId fileinfo_root;
+    wxString fileinfo_root_label;
+
 };
 
 #if !defined(__WXX11__)
@@ -422,38 +507,20 @@ private:
 };
 #endif
 
-/* Popup contextual menu */
-class PopupMenu: public wxMenu
+/* Menus */
+void PopupMenu( intf_thread_t *_p_intf, wxWindow *p_parent,
+                const wxPoint& pos );
+wxMenu *AudioMenu( intf_thread_t *_p_intf, wxWindow *p_parent );
+wxMenu *VideoMenu( intf_thread_t *_p_intf, wxWindow *p_parent );
+wxMenu *NavigMenu( intf_thread_t *_p_intf, wxWindow *p_parent );
+
+class MenuEvtHandler : public wxEvtHandler
 {
 public:
-    /* Constructor */
-    PopupMenu( intf_thread_t *p_intf, Interface *p_main_interface );
-    virtual ~PopupMenu();
+    MenuEvtHandler( intf_thread_t *p_intf, Interface *p_main_interface );
+    virtual ~MenuEvtHandler();
 
-private:
-    /* Event handlers (these functions should _not_ be virtual) */
-    void OnClose( wxCommandEvent& event );
-    void OnEntrySelected( wxCommandEvent& event );
-
-    wxMenu *PopupMenu::CreateDummyMenu();
-    void   PopupMenu::CreateMenuEntry( char *, vlc_object_t * );
-    wxMenu *PopupMenu::CreateSubMenu( char *, vlc_object_t * );
-
-    DECLARE_EVENT_TABLE();
-
-    intf_thread_t *p_intf;
-    Interface *p_main_interface;
-
-    int  i_item_id;
-};
-
-class PopupEvtHandler : public wxEvtHandler
-{
-public:
-    PopupEvtHandler( intf_thread_t *p_intf, Interface *p_main_interface );
-    virtual ~PopupEvtHandler();
-
-    void PopupEvtHandler::OnMenuEvent( wxCommandEvent& event );
+    void MenuEvtHandler::OnMenuEvent( wxCommandEvent& event );
 
 private:
 
@@ -463,29 +530,26 @@ private:
     Interface *p_main_interface;
 };
 
-class wxMenuItemExt: public wxMenuItem
+class Menu: public wxMenu
 {
 public:
     /* Constructor */
-    wxMenuItemExt( wxMenu* parentMenu, int id,
-                   const wxString& text,
-                   const wxString& helpString,
-                   wxItemKind kind,
-                   char *_psz_var, int _i_object_id, vlc_value_t _val ):
-        wxMenuItem( parentMenu, id, text, helpString, kind )
-    {
-        /* Initializations */
-        psz_var = _psz_var;
-        i_object_id = _i_object_id;
-        val = _val;
-    };
-
-    virtual ~wxMenuItemExt() { if( psz_var ) free( psz_var ); };
-
-    char *psz_var;
-    int  i_object_id;
-    vlc_value_t val;
+    Menu( intf_thread_t *p_intf, wxWindow *p_parent, int i_count,
+          char **ppsz_names, int *pi_objects, int i_start_id );
+    virtual ~Menu();
 
 private:
+    /* Event handlers (these functions should _not_ be virtual) */
+    void OnClose( wxCommandEvent& event );
+    void OnEntrySelected( wxCommandEvent& event );
 
+    wxMenu *Menu::CreateDummyMenu();
+    void   Menu::CreateMenuItem( wxMenu *, char *, vlc_object_t * );
+    wxMenu *Menu::CreateChoicesMenu( char *, vlc_object_t * );
+
+    DECLARE_EVENT_TABLE();
+
+    intf_thread_t *p_intf;
+
+    int  i_item_id;
 };
