@@ -2,7 +2,7 @@
  * sap.c :  SAP interface module
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: sap.c,v 1.10 2003/03/30 18:14:38 gbazin Exp $
+ * $Id: sap.c,v 1.15 2003/06/20 11:42:07 alexis Exp $
  *
  * Authors: Arnaud Schauly <gitan@via.ecp.fr>
  *
@@ -62,6 +62,12 @@
 #   endif
 #endif
 
+#ifdef UNDER_CE
+#   define close(a) CloseHandle(a);
+#elif defined( WIN32 )
+#   define close(a) closesocket(a);
+#endif
+
 #include "network.h"
 
 #define MAX_LINE_LENGTH 256
@@ -71,6 +77,9 @@
 #define HELLO_GROUP "224.2.127.254"
 #define ADD_SESSION 1
 
+#define IPV6_ADDR_1 "FF0"  /* Scope is inserted between them */
+#define IPV6_ADDR_2 "::2:7FFE"
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -82,14 +91,14 @@ static int  Activate     ( vlc_object_t * );
 static void Run          ( intf_thread_t *p_intf );
 static int Kill          ( intf_thread_t * );
 
-static ssize_t NetRead    ( intf_thread_t*, int , byte_t *, size_t );
+static ssize_t NetRead    ( intf_thread_t*, int, int , byte_t *, size_t );
 
 /* playlist related functions */
 static int  sess_toitem( intf_thread_t *, sess_descr_t * );
 
 /* sap/sdp related functions */
 static int parse_sap ( char * );
-static int packet_handle ( intf_thread_t *, char * );
+static int packet_handle ( intf_thread_t *, char *, int );
 static sess_descr_t *  parse_sdp( intf_thread_t *, char * ) ;
 
 /* specific sdp fields parsing */
@@ -131,6 +140,22 @@ vlc_module_begin();
     add_category_hint( N_("SAP"), NULL, VLC_TRUE );
         add_string( "sap-addr", NULL, NULL,
                      "SAP multicast address", "SAP multicast address", VLC_TRUE );
+    
+        add_bool( "no-sap-ipv4", 0 , NULL,
+                     "NO IPv4-SAP listening",
+                     "Set this if you do not want SAP to listen for IPv4 announces",
+                     VLC_TRUE);
+
+        add_bool( "sap-ipv6", 0 , NULL,
+                   "IPv6-SAP listening", 
+                   "Set this if you want SAP to listen for IPv6 announces",
+                   VLC_TRUE);
+
+        add_string( "sap-ipv6-scope", "8" , NULL,
+                    "IPv6-SAP scope",
+                    "Sets the scope for IPv6 announces (default is 8)",
+                    VLC_TRUE);
+
     set_description( _("SAP interface") );
     set_capability( "interface", 0 );
     set_callbacks( Activate, NULL);
@@ -158,41 +183,89 @@ static int Activate( vlc_object_t *p_this )
 static void Run( intf_thread_t *p_intf )
 {
     char *psz_addr;
+    char *psz_addrv6;
     char *psz_network = NULL;
-    int fd;
-
+    int fd            = - 1;
+    int fdv6          = -1;
+    
+    int no_sap_ipv4      = config_GetInt( p_intf, "no-sap-ipv4" );
+    int sap_ipv6         = config_GetInt( p_intf, "sap-ipv6" );
+    char *sap_ipv6_scope = config_GetPsz( p_intf, "sap-ipv6-scope" );
+    
     char buffer[MAX_SAP_BUFFER + 1];
 
     module_t            *p_network;
     network_socket_t    socket_desc;
 
-    if( !(psz_addr = config_GetPsz( p_intf, "sap-addr" ) ) )
+    if( no_sap_ipv4 == -1 || sap_ipv6 == -1 || sap_ipv6_scope == NULL )
     {
-        psz_addr = strdup( HELLO_GROUP );
-    }
-
-    /* Prepare the network_socket_t structure */
-    socket_desc.i_type = NETWORK_UDP;
-    socket_desc.psz_bind_addr = psz_addr;
-    socket_desc.i_bind_port   = HELLO_PORT;
-    socket_desc.psz_server_addr   = "";
-    socket_desc.i_server_port     = 0;
-    p_intf->p_private = (void*) &socket_desc;
-
-    psz_network = "ipv4"; // FIXME
-
-    /* Create, Bind the socket, ... with the appropriate module  */
-
-    if( !( p_network = module_Need( p_intf, "network", psz_network ) ) )
-    {
-        msg_Warn( p_intf, "failed to open a connection (udp)" );
+        msg_Warn( p_intf, "Unable to parse module configuration" );
         return;
     }
-    module_Unneed( p_intf, p_network );
+    
+    /* Prepare IPv4 Networking */
+    if ( no_sap_ipv4 == 0)
+    {
+        if( !(psz_addr = config_GetPsz( p_intf, "sap-addr" ) ) )
+        { 
+            psz_addr = strdup( HELLO_GROUP );
+        } 
+            
+        /* Prepare the network_socket_t structure */
+        socket_desc.i_type            = NETWORK_UDP;
+        socket_desc.psz_bind_addr     = psz_addr;
+        socket_desc.i_bind_port       = HELLO_PORT;
+        socket_desc.psz_server_addr   = "";
+        socket_desc.i_server_port     = 0;
+        p_intf->p_private = (void*) &socket_desc;
 
-    fd = socket_desc.i_handle;
+        psz_network = "ipv4"; 
 
+       /* Create, Bind the socket, ... with the appropriate module  */
+ 
+        if( !( p_network = module_Need( p_intf, "network", psz_network ) ) )
+        {
+            msg_Warn( p_intf, "failed to open a connection (udp)" );
+            return;
+        }
+        module_Unneed( p_intf, p_network );
 
+        fd = socket_desc.i_handle;
+    }
+
+    /* Prepare IPv6 Networking */
+    if ( sap_ipv6 > 0)
+    {
+        /* Prepare the network_socket_t structure */
+
+        psz_addrv6=(char *)malloc(sizeof(char)*38); 
+        /* Max size of an IPv6 address */
+        
+        sprintf(psz_addrv6,"[%s%c%s]",IPV6_ADDR_1,
+                        sap_ipv6_scope[0],IPV6_ADDR_2);
+        
+        socket_desc.i_type            = NETWORK_UDP;
+        socket_desc.psz_bind_addr     = psz_addrv6;
+        socket_desc.i_bind_port       = HELLO_PORT;
+        socket_desc.psz_server_addr   = "";
+        socket_desc.i_server_port     = 0;
+        p_intf->p_private = (void*) &socket_desc;
+
+        psz_network = "ipv6"; 
+
+       /* Create, Bind the socket, ... with the appropriate module  */
+ 
+        if( !( p_network = module_Need( p_intf, "network", psz_network ) ) )
+        {
+            msg_Warn( p_intf, "failed to open a connection (udp)" );
+            return;
+        }
+        module_Unneed( p_intf, p_network );
+
+        fdv6 = socket_desc.i_handle;
+    }
+
+    
     /* read SAP packets */
     while( !p_intf->b_die )
     {
@@ -200,7 +273,7 @@ static void Run( intf_thread_t *p_intf )
 
         //memset( buffer, 0, MAX_SAP_BUFFER + 1);
 
-        i_read = NetRead( p_intf, fd, buffer, MAX_SAP_BUFFER );
+        i_read = NetRead( p_intf, fd, fdv6, buffer, MAX_SAP_BUFFER );
 
         if( i_read < 0 )
         {
@@ -212,20 +285,12 @@ static void Run( intf_thread_t *p_intf )
         }
         buffer[i_read] = '\0';
 
-        packet_handle( p_intf, buffer );
+        packet_handle( p_intf, buffer, i_read );
 
     }
 
     /* Closing socket */
-
-#ifdef UNDER_CE
-    CloseHandle( socket_desc.i_handle );
-#elif defined( WIN32 )
-    closesocket( socket_desc.i_handle );
-#else
     close( socket_desc.i_handle );
-#endif
-
 }
 
 /********************************************************************
@@ -451,12 +516,20 @@ static void mfield_parse( char *psz_mfield, char **ppsz_proto,
  * parse_sap : Takes care of the SAP headers
  ***********************************************************************
  * checks if the packet has the true headers ;
+ * returns the SAP header lenhth
  ***********************************************************************/
 
 static int parse_sap( char *p_packet )
-{  /* Dummy Parser : does nothing !*/
+{
+    // According to RFC 2974
+    int i_hlen = 4;                           // Minimum header length is 4
+    i_hlen += (p_packet[0] & 0x10) ? 16 : 4;  // Address type IPv6=16bytes
+    i_hlen +=  p_packet[1];                   // Authentification length
 
-    return( VLC_TRUE );
+    //Looks for the first '\0' byte after length
+    for(;p_packet[i_hlen]!='\0'; i_hlen++);
+
+    return(i_hlen);
 }
 
 /*************************************************************************
@@ -464,18 +537,22 @@ static int parse_sap( char *p_packet )
  * the understated session
  *************************************************************************/
 
-static int packet_handle( intf_thread_t * p_intf, char *p_packet )
+static int packet_handle( intf_thread_t * p_intf, char *p_packet, int i_len )
 {
     sess_descr_t * p_sd;
+    int i_hlen;                             // Header length
 
-    if( parse_sap( p_packet ) )
+    i_hlen = parse_sap(p_packet);
+
+    if( (i_hlen > 0) && (i_hlen < i_len) )
     {
-        p_sd = parse_sdp( p_intf, p_packet);
-
-        sess_toitem ( p_intf, p_sd );
-
-        free_sd ( p_sd );
-        return VLC_TRUE;
+        p_sd = parse_sdp( p_intf, p_packet + i_hlen +1);
+        if(p_sd)
+        {
+            sess_toitem ( p_intf, p_sd );
+            free_sd ( p_sd );
+            return VLC_TRUE;
+        }
     }
 
     return VLC_FALSE; // Invalid Packet
@@ -494,6 +571,13 @@ static sess_descr_t *  parse_sdp( intf_thread_t * p_intf, char *p_packet )
 {
     sess_descr_t *  sd;
 
+    // According to RFC 2327, the first bytes should be exactly "v="
+    if((p_packet[0] != 'v') || (p_packet[1] != '='))
+    {
+        msg_Warn(p_intf, "Bad SDP packet");
+        return NULL;
+    }
+
     if( ( sd = malloc( sizeof(sess_descr_t) ) ) == NULL )
     {
         msg_Err( p_intf, "Not enough memory for sd in parse_sdp()" );
@@ -514,7 +598,6 @@ static sess_descr_t *  parse_sdp( intf_thread_t * p_intf, char *p_packet )
 
     sd->i_media = 0;
 
-
     while( *p_packet != '\0'  )
     {
 #define FIELD_COPY( p ) \
@@ -578,7 +661,7 @@ static sess_descr_t *  parse_sdp( intf_thread_t * p_intf, char *p_packet )
                     {
                         sd->pp_media =
                             realloc( sd->pp_media,
-                                     sizeof( media_descr_t ) * ( sd->i_media + 1 ) );
+                               sizeof( media_descr_t ) * ( sd->i_media + 1 ) );
                     }
                     else
                     {
@@ -597,11 +680,11 @@ static sess_descr_t *  parse_sdp( intf_thread_t * p_intf, char *p_packet )
                 case( 'c' ):
                     if( sd->i_media <= 0 )
                     {
-                        sd->psz_connection = strndup( &p_packet[2], i_field_len );
+                        FIELD_COPY(sd->psz_connection);
                     }
                     else
                     {
-                        sd->pp_media[sd->i_media - 1]->psz_mediaconnection = strndup( &p_packet[2], i_field_len );
+                        FIELD_COPY(sd->pp_media[sd->i_media - 1]->psz_mediaconnection);
                     }
                    break;
                 default:
@@ -650,7 +733,7 @@ static void free_sd( sess_descr_t * p_sd )
 }
 
 /***********************************************************************
- * ismult
+ * ismult: returns true if we have a multicast address
  ***********************************************************************/
 
 static int ismult( char *psz_uri )
@@ -660,10 +743,10 @@ static int ismult( char *psz_uri )
 
     i_value = strtol( psz_uri, &psz_end, 0 );
 
-    if( *psz_end != '.' )
-    {
-        return( VLC_FALSE );
-    }
+    /* FIXME: This is an ugly way to detect IPv6 multicast */
+    if( psz_uri[0] == '[') { return( VLC_TRUE ); } 
+
+    if( *psz_end != '.' ) { return( VLC_FALSE ); }
 
     return( i_value < 224 ? VLC_FALSE : VLC_TRUE );
 }
@@ -674,7 +757,8 @@ static int ismult( char *psz_uri )
  * Taken from udp.c
  ******************************************************************************/
 static ssize_t NetRead( intf_thread_t *p_intf,
-                        int i_handle, byte_t *p_buffer, size_t i_len)
+                        int i_handle, int i_handle_v6, 
+                        byte_t *p_buffer, size_t i_len)
 {
 #ifdef UNDER_CE
     return -1;
@@ -683,17 +767,29 @@ static ssize_t NetRead( intf_thread_t *p_intf,
     struct timeval  timeout;
     fd_set          fds;
     int             i_ret;
+    int             i_max_handle;
 
+    ssize_t i_recv=-1;
+    
+    /* Get the max handle for select */
+    if( i_handle_v6 > i_handle )
+        i_max_handle = i_handle_v6;
+    else
+        i_max_handle = i_handle;
+
+    
     /* Initialize file descriptor set */
     FD_ZERO( &fds );
-    FD_SET( i_handle, &fds );
+    if(   i_handle > 0   ) FD_SET( i_handle, &fds );
+    if( i_handle_v6  > 0 ) FD_SET( i_handle_v6, &fds);
+
 
     /* We'll wait 0.5 second if nothing happens */
     timeout.tv_sec = 0;
     timeout.tv_usec = 500000;
 
     /* Find if some data is available */
-    i_ret = select( i_handle + 1, &fds,
+    i_ret = select( i_max_handle + 1, &fds,
     NULL, NULL, &timeout );
 
     if( i_ret == -1 && errno != EINTR )
@@ -701,14 +797,27 @@ static ssize_t NetRead( intf_thread_t *p_intf,
         msg_Err( p_intf, "network select error (%s)", strerror(errno) );
     }
     else if( i_ret > 0 )
-    {
-        ssize_t i_recv = recv( i_handle, p_buffer, i_len, 0 );
-
-        if( i_recv < 0 )
+   {
+      /* Get the data */     
+      if(i_handle >0)
+      {
+         if(FD_ISSET( i_handle, &fds ))
+         {
+             i_recv = recv( i_handle, p_buffer, i_len, 0 );
+         }
+      }
+      if(i_handle_v6 >0)
+      {
+         if(FD_ISSET( i_handle_v6, &fds ))
+         {
+            i_recv = recv( i_handle_v6, p_buffer, i_len, 0 );
+         }
+      }
+      
+       if( i_recv < 0 )
         {
            msg_Err( p_intf, "recv failed (%s)", strerror(errno) );
         }
-
         return i_recv;
     }
 
