@@ -2,7 +2,7 @@
  * xmga.c : X11 MGA plugin for vlc
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: xmga.c,v 1.4 2003/03/30 18:14:39 gbazin Exp $
+ * $Id: xmga.c,v 1.6 2003/05/15 22:27:38 massiot Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -52,8 +52,6 @@
 #include <X11/extensions/XShm.h>
 #include <X11/extensions/dpms.h>
 
-#include "netutils.h"                                 /* network_ChannelJoin */
-
 //#include "mga.h"
 
 /*****************************************************************************
@@ -86,7 +84,7 @@ static void ToggleCursor   ( vout_thread_t * );
  * Module descriptor
  *****************************************************************************/
 
-#define ALT_FS_TEXT N_("alternate fullscreen method")
+#define ALT_FS_TEXT N_("Alternate fullscreen method")
 #define ALT_FS_LONGTEXT N_( \
     "There are two ways to make a fullscreen window, unfortunately each one " \
     "has its drawbacks.\n" \
@@ -469,19 +467,6 @@ static int Manage( vout_thread_t *p_vout )
                                  INPUT_STATUS_PAUSE );
                 break;
 
-            case XK_F1: network_ChannelJoin( p_vout, 1 ); break;
-            case XK_F2: network_ChannelJoin( p_vout, 2 ); break;
-            case XK_F3: network_ChannelJoin( p_vout, 3 ); break;
-            case XK_F4: network_ChannelJoin( p_vout, 4 ); break;
-            case XK_F5: network_ChannelJoin( p_vout, 5 ); break;
-            case XK_F6: network_ChannelJoin( p_vout, 6 ); break;
-            case XK_F7: network_ChannelJoin( p_vout, 7 ); break;
-            case XK_F8: network_ChannelJoin( p_vout, 8 ); break;
-            case XK_F9: network_ChannelJoin( p_vout, 9 ); break;
-            case XK_F10: network_ChannelJoin( p_vout, 10 ); break;
-            case XK_F11: network_ChannelJoin( p_vout, 11 ); break;
-            case XK_F12: network_ChannelJoin( p_vout, 12 ); break;
-
             default:
                 /* "Normal Keys"
                  * The reason why I use this instead of XK_0 is that 
