@@ -4,7 +4,7 @@
  * decoders.
  *****************************************************************************
  * Copyright (C) 1998-2002 VideoLAN
- * $Id: input.c,v 1.228 2003/03/30 01:13:37 massiot Exp $
+ * $Id: input.c,v 1.230 2003/05/22 16:01:02 gbazin Exp $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -36,7 +36,6 @@
 #   include <sys/times.h>
 #endif
 
-#include "netutils.h"
 #include "vlc_playlist.h"
 
 #include "stream_control.h"
@@ -489,6 +488,11 @@ static int InitThread( input_thread_t * p_input )
         p_input->i_bufsize = INPUT_DEFAULT_BUFSIZE;
     }
 
+    /* If the desynchronisation requested by the user is < 0, we need to
+     * cache more data. */
+    if( p_input->p_vlc->i_desync < 0 )
+        p_input->i_pts_delay -= p_input->p_vlc->i_desync;
+
     if( p_input->p_current_data == NULL && p_input->pf_read != NULL )
     {
         while( !input_FillBuffer( p_input ) )
