@@ -2,7 +2,7 @@
  * vout_pictures.c : picture management functions
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
- * $Id: vout_pictures.c,v 1.36 2003/03/28 17:02:25 gbazin Exp $
+ * $Id: vout_pictures.c,v 1.40 2003/06/09 00:33:34 massiot Exp $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -108,7 +108,7 @@ void vout_DatePicture( vout_thread_t *p_vout,
 picture_t *vout_CreatePicture( vout_thread_t *p_vout,
                                vlc_bool_t b_progressive,
                                vlc_bool_t b_top_field_first,
-                               vlc_bool_t b_repeat_first_field )
+                               unsigned int i_nb_fields )
 {
     int         i_pic;                                      /* picture index */
     picture_t * p_pic;
@@ -136,7 +136,7 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout,
                 p_pic->b_force    = 0;
 
                 p_pic->b_progressive        = b_progressive;
-                p_pic->b_repeat_first_field = b_repeat_first_field;
+                p_pic->i_nb_fields          = i_nb_fields;
                 p_pic->b_top_field_first    = b_top_field_first;
 
                 p_vout->i_heap_size++;
@@ -178,7 +178,7 @@ picture_t *vout_CreatePicture( vout_thread_t *p_vout,
             p_freepic->b_force = 0;
 
             p_freepic->b_progressive        = b_progressive;
-            p_freepic->b_repeat_first_field = b_repeat_first_field;
+            p_freepic->i_nb_fields          = i_nb_fields;
             p_freepic->b_top_field_first    = b_top_field_first;
 
             p_freepic->i_matrix_coefficients = 1;
@@ -259,7 +259,8 @@ void vout_UnlinkPicture( vout_thread_t *p_vout, picture_t *p_pic )
 
     if( p_pic->i_refcount < 0 )
     {
-        msg_Err( p_vout, "picture refcount is %i", p_pic->i_refcount );
+        msg_Err( p_vout, "picture %p refcount is %i", 
+                 p_pic, p_pic->i_refcount );
         p_pic->i_refcount = 0;
     }
 
@@ -437,6 +438,44 @@ void vout_AllocatePicture( vout_thread_t *p_vout, picture_t *p_pic,
 {
     int i_bytes, i_index;
 
+    vout_InitPicture( VLC_OBJECT(p_vout), p_pic, i_width, i_height, i_chroma );
+
+    /* Calculate how big the new image should be */
+    for( i_bytes = 0, i_index = 0; i_index < p_pic->i_planes; i_index++ )
+    {
+        i_bytes += p_pic->p[ i_index ].i_lines * p_pic->p[ i_index ].i_pitch;
+    }
+
+    p_pic->p_data = vlc_memalign( &p_pic->p_data_orig, 16, i_bytes );
+
+    if( p_pic->p_data == NULL )
+    {
+        p_pic->i_planes = 0;
+        return;
+    }
+
+    /* Fill the p_pixels field for each plane */
+    p_pic->p[ 0 ].p_pixels = p_pic->p_data;
+
+    for( i_index = 1; i_index < p_pic->i_planes; i_index++ )
+    {
+        p_pic->p[i_index].p_pixels = p_pic->p[i_index-1].p_pixels
+                                          + p_pic->p[i_index-1].i_lines
+                                             * p_pic->p[i_index-1].i_pitch;
+    }
+}
+
+/*****************************************************************************
+ * vout_InitPicture: initialise the picture_t fields given chroma/size.
+ *****************************************************************************
+ * This function initializes most of the picture_t fields given a chroma and
+ * size. It makes the assumption that stride == width.
+ *****************************************************************************/
+void vout_InitPicture( vlc_object_t *p_this, picture_t *p_pic,
+                       int i_width, int i_height, vlc_fourcc_t i_chroma )
+{
+    int i_index;
+
     /* Store default values */
     for( i_index = 0; i_index < VOUT_MAX_PLANES; i_index++ )
     {
@@ -447,6 +486,19 @@ void vout_AllocatePicture( vout_thread_t *p_vout, picture_t *p_pic,
     /* Calculate coordinates */
     switch( i_chroma )
     {
+        case FOURCC_I411:
+            p_pic->p[ Y_PLANE ].i_lines = i_height;
+            p_pic->p[ Y_PLANE ].i_pitch = i_width;
+            p_pic->p[ Y_PLANE ].i_visible_pitch = p_pic->p[ Y_PLANE ].i_pitch;
+            p_pic->p[ U_PLANE ].i_lines = i_height;
+            p_pic->p[ U_PLANE ].i_pitch = i_width / 4;
+            p_pic->p[ U_PLANE ].i_visible_pitch = p_pic->p[ U_PLANE ].i_pitch;
+            p_pic->p[ V_PLANE ].i_lines = i_height;
+            p_pic->p[ V_PLANE ].i_pitch = i_width / 4;
+            p_pic->p[ V_PLANE ].i_visible_pitch = p_pic->p[ V_PLANE ].i_pitch;
+            p_pic->i_planes = 3;
+            break;
+
         case FOURCC_I410:
             p_pic->p[ Y_PLANE ].i_lines = i_height;
             p_pic->p[ Y_PLANE ].i_pitch = i_width;
@@ -574,35 +626,12 @@ void vout_AllocatePicture( vout_thread_t *p_vout, picture_t *p_pic,
             break;
 
         default:
-            msg_Err( p_vout, "unknown chroma type 0x%.8x (%4.4s)",
+            msg_Err( p_this, "unknown chroma type 0x%.8x (%4.4s)",
                              i_chroma, (char*)&i_chroma );
             p_pic->i_planes = 0;
             return;
     }
 
-    /* Calculate how big the new image should be */
-    for( i_bytes = 0, i_index = 0; i_index < p_pic->i_planes; i_index++ )
-    {
-        i_bytes += p_pic->p[ i_index ].i_lines * p_pic->p[ i_index ].i_pitch;
-    }
-
-    p_pic->p_data = vlc_memalign( &p_pic->p_data_orig, 16, i_bytes );
-
-    if( p_pic->p_data == NULL )
-    {
-        p_pic->i_planes = 0;
-        return;
-    }
-
-    /* Fill the p_pixels field for each plane */
-    p_pic->p[ 0 ].p_pixels = p_pic->p_data;
-
-    for( i_index = 1; i_index < p_pic->i_planes; i_index++ )
-    {
-        p_pic->p[i_index].p_pixels = p_pic->p[i_index-1].p_pixels
-                                          + p_pic->p[i_index-1].i_lines
-                                             * p_pic->p[i_index-1].i_pitch;
-    }
 }
 
 /*****************************************************************************
